@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,20 +12,20 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Helper functions for adding files to working trees."""
 
-import errno
-import os
+from __future__ import absolute_import
+
 import sys
+import os
 
-import bzrlib.bzrdir
-import bzrlib.errors as errors
-import bzrlib.osutils
-from bzrlib.symbol_versioning import *
-from bzrlib.workingtree import WorkingTree
-
+from bzrlib import (
+    osutils,
+    ui, 
+    )
+from bzrlib.i18n import gettext
 
 class AddAction(object):
     """A class which defines what action to take when adding a file."""
@@ -35,7 +35,7 @@ class AddAction(object):
 
         :param to_file: The stream to write into. This is expected to take
             Unicode paths. If not supplied, it will default to ``sys.stdout``.
-        :param should_print: If False, printing will be supressed.
+        :param should_print: If False, printing will be suppressed.
         """
         self._to_file = to_file
         if to_file is None:
@@ -44,7 +44,7 @@ class AddAction(object):
         if should_print is not None:
             self.should_print = should_print
 
-    def __call__(self, inv, parent_ie, path, kind, _quote=bzrlib.osutils.quotefn):
+    def __call__(self, inv, parent_ie, path, kind, _quote=osutils.quotefn):
         """Add path to inventory.
 
         The default action does nothing.
@@ -54,9 +54,49 @@ class AddAction(object):
         :param kind: The kind of the object being added.
         """
         if self.should_print:
-            self._to_file.write('adding %s\n' % _quote(path.raw_path))
+            self._to_file.write('adding %s\n' % _quote(path))
         return None
 
+    def skip_file(self, tree, path, kind, stat_value = None):
+        """Test whether the given file should be skipped or not.
+        
+        The default action never skips. Note this is only called during
+        recursive adds
+        
+        :param tree: The tree we are working in
+        :param path: The path being added
+        :param kind: The kind of object being added.
+        :param stat: Stat result for this file, if available already
+        :return bool. True if the file should be skipped (not added)
+        """
+        return False
+
+
+class AddWithSkipLargeAction(AddAction):
+    """A class that can decide to skip a file if it's considered too large"""
+
+    _maxSize = None
+
+    def skip_file(self, tree, path, kind, stat_value = None):
+        if kind != 'file':
+            return False
+        opt_name = 'add.maximum_file_size'
+        if self._maxSize is None:
+            # FIXME: We use the branch config as there is no tree config
+            # -- vila 2011-12-16
+            config = tree.branch.get_config_stack()
+            self._maxSize = config.get(opt_name)
+        if stat_value is None:
+            file_size = os.path.getsize(path);
+        else:
+            file_size = stat_value.st_size;
+        if self._maxSize > 0 and file_size > self._maxSize:
+            ui.ui_factory.show_warning(gettext(
+                "skipping {0} (larger than {1} of {2} bytes)").format(
+                path, opt_name,  self._maxSize))
+            return True
+        return False
+
 
 class AddFromBaseAction(AddAction):
     """This class will try to extract file ids from another tree."""
@@ -74,7 +114,7 @@ class AddFromBaseAction(AddAction):
         if file_id is not None:
             if self.should_print:
                 self._to_file.write('adding %s w/ file id from %s\n'
-                                    % (path.raw_path, base_path))
+                                    % (path, base_path))
         else:
             # we aren't doing anything special, so let the default
             # reporter happen
@@ -90,23 +130,13 @@ class AddFromBaseAction(AddAction):
         Else, we look for an entry in the base tree with the same path.
         """
 
-        if (parent_ie.file_id in self.base_tree):
+        if self.base_tree.has_id(parent_ie.file_id):
             base_parent_ie = self.base_tree.inventory[parent_ie.file_id]
-            base_child_ie = base_parent_ie.children.get(path.base_path)
+            base_child_ie = base_parent_ie.children.get(
+                osutils.basename(path))
             if base_child_ie is not None:
                 return (base_child_ie.file_id,
                         self.base_tree.id2path(base_child_ie.file_id))
-        full_base_path = bzrlib.osutils.pathjoin(self.base_path, path.raw_path)
+        full_base_path = osutils.pathjoin(self.base_path, path)
         # This may return None, but it is our last attempt
         return self.base_tree.path2id(full_base_path), full_base_path
-
-
-# TODO: jam 20050105 These could be used for compatibility
-#       however, they bind against the current stdout, not the
-#       one which exists at the time they are called, so they
-#       don't work for the test suite.
-# deprecated
-add_action_add = AddAction()
-add_action_null = add_action_add
-add_action_add_and_print = AddAction(should_print=True)
-add_action_print = add_action_add_and_print
