@@ -14,37 +14,31 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 
-from bzrlib.symbol_versioning import deprecated_function, deprecated_in
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
     errors,
+    transport as _mod_transport,
     urlutils,
     )
 from bzrlib.bundle import serializer as _serializer
 from bzrlib.merge_directive import MergeDirective
-from bzrlib.transport import (
-    do_catching_redirections,
-    get_transport,
-    )
+from bzrlib.i18n import gettext
 """)
 from bzrlib.trace import note
 
 
-@deprecated_function(deprecated_in((1, 12, 0)))
-def read_bundle_from_url(url):
-    return read_mergeable_from_url(url, _do_directive=False)
-
-
 def read_mergeable_from_url(url, _do_directive=True, possible_transports=None):
     """Read mergable object from a given URL.
 
     :return: An object supporting get_target_revision.  Raises NotABundle if
         the target is not a mergeable type.
     """
-    child_transport = get_transport(url,
+    child_transport = _mod_transport.get_transport(url,
         possible_transports=possible_transports)
     transport = child_transport.clone('..')
     filename = transport.relpath(child_transport.base)
@@ -62,12 +56,12 @@ def read_mergeable_from_transport(transport, filename, _do_directive=True):
         url, filename = urlutils.split(exception.target,
                                        exclude_trailing_slash=False)
         if not filename:
-            raise errors.NotABundle('A directory cannot be a bundle')
-        return get_transport(url)
+            raise errors.NotABundle(gettext('A directory cannot be a bundle'))
+        return _mod_transport.get_transport_from_url(url)
 
     try:
-        bytef, transport = do_catching_redirections(get_bundle, transport,
-                                                    redirected_transport)
+        bytef, transport = _mod_transport.do_catching_redirections(
+            get_bundle, transport, redirected_transport)
     except errors.TooManyRedirections:
         raise errors.NotABundle(transport.clone(filename).base)
     except (errors.ConnectionReset, errors.ConnectionError), e:
