@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,24 +17,23 @@
 """Serializer factory for reading and writing bundles.
 """
 
-import os
+from __future__ import absolute_import
 
-from bzrlib import errors
+from bzrlib import (
+    errors,
+    ui,
+    )
 from bzrlib.bundle.serializer import (BundleSerializer,
                                       _get_bundle_header,
                                      )
 from bzrlib.bundle.serializer import binary_diff
-from bzrlib.bundle.bundle_data import (RevisionInfo, BundleInfo, BundleTree)
+from bzrlib.bundle.bundle_data import (RevisionInfo, BundleInfo)
 from bzrlib.diff import internal_diff
-from bzrlib.osutils import pathjoin
-from bzrlib.progress import DummyProgress
 from bzrlib.revision import NULL_REVISION
-import bzrlib.ui
 from bzrlib.testament import StrictTestament
 from bzrlib.timestamp import (
     format_highres_date,
-    unpack_highres_date,
-)
+    )
 from bzrlib.textfile import text_file
 from bzrlib.trace import mutter
 
@@ -119,12 +118,11 @@ class BundleSerializerV08(BundleSerializer):
         source.lock_read()
         try:
             self._write_main_header()
-            pb = DummyProgress()
+            pb = ui.ui_factory.nested_progress_bar()
             try:
                 self._write_revisions(pb)
             finally:
-                pass
-                #pb.finished()
+                pb.finished()
         finally:
             source.unlock()
 
@@ -183,7 +181,7 @@ class BundleSerializerV08(BundleSerializer):
 
         i_max = len(self.revision_ids)
         for i, rev_id in enumerate(self.revision_ids):
-            pb.update("Generating revsion data", i, i_max)
+            pb.update("Generating revision data", i, i_max)
             rev = self.source.get_revision(rev_id)
             if rev_id == last_rev_id:
                 rev_tree = last_rev_tree
@@ -263,7 +261,7 @@ class BundleSerializerV08(BundleSerializer):
 
         def do_diff(file_id, old_path, new_path, action, force_binary):
             def tree_lines(tree, require_text=False):
-                if file_id in tree:
+                if tree.has_id(file_id):
                     tree_file = tree.get_file(file_id)
                     if require_text is True:
                         tree_file = text_file(tree_file)
@@ -553,5 +551,5 @@ class BundleInfo08(BundleInfo):
         testament = StrictTestament.from_revision(repository, revision_id)
         return testament.as_sha1()
 
-    def _testament_sha1(self, revision, inventory):
-        return StrictTestament(revision, inventory).as_sha1()
+    def _testament_sha1(self, revision, tree):
+        return StrictTestament(revision, tree).as_sha1()
