@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib.bundle.serializer import _get_bundle_header
 from bzrlib.bundle.serializer.v08 import BundleSerializerV08, BundleReader
 from bzrlib.testament import StrictTestament3
@@ -63,8 +65,8 @@ class BundleInfo09(BundleInfo):
         testament = StrictTestament3.from_revision(repository, revision_id)
         return testament.as_sha1()
 
-    def _testament_sha1(self, revision, inventory):
-        return StrictTestament3(revision, inventory).as_sha1()
+    def _testament_sha1(self, revision, tree):
+        return StrictTestament3(revision, tree).as_sha1()
 
 
 class BundleReaderV09(BundleReader):
