@@ -24,15 +24,16 @@ rather starts again from the run_bzr function.
 """
 
 
-import os
+from bzrlib import (
+    config,
+    tests,
+    )
 
-from bzrlib.branch import Branch
-from bzrlib.config import extract_email_address
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.urlutils import joinpath
 
 
-class TestAnnotate(TestCaseWithTransport):
+class TestAnnotate(tests.TestCaseWithTransport):
 
     def setUp(self):
         super(TestAnnotate, self).setUp()
@@ -140,7 +141,7 @@ class TestAnnotate(TestCaseWithTransport):
         out, err = self.run_bzr('annotate hello.txt -r 10',
                                 retcode=3)
         self.assertEqual('', out)
-        self.assertContainsRe(err, 'Requested revision: \'10\' does not exist')
+        self.assertContainsRe(err, "Requested revision: '10' does not exist")
 
     def test_annotate_cmd_two_revisions(self):
         out, err = self.run_bzr('annotate hello.txt -r1..2',
@@ -151,7 +152,7 @@ class TestAnnotate(TestCaseWithTransport):
                          err)
 
 
-class TestSimpleAnnotate(TestCaseWithTransport):
+class TestSimpleAnnotate(tests.TestCaseWithTransport):
     """Annotate tests with no complex setup."""
 
     def _setup_edited_file(self, relpath='.'):
@@ -162,14 +163,13 @@ class TestSimpleAnnotate(TestCaseWithTransport):
         tree.add('file')
         tree.commit('add file', committer="test@host", rev_id="rev1")
         self.build_tree_contents([(file_relpath, 'foo\nbar\ngam\n')])
-        tree.branch.get_config().set_user_option('email', 'current@host2')
         return tree
 
     def test_annotate_cmd_revspec_branch(self):
         tree = self._setup_edited_file('trunk')
         tree.branch.create_checkout(self.get_url('work'), lightweight=True)
-        os.chdir('work')
-        out, err = self.run_bzr('annotate file -r branch:../trunk')
+        out, err = self.run_bzr(['annotate', 'file', '-r', 'branch:../trunk'],
+                                working_dir='work')
         self.assertEqual('', err)
         self.assertEqual(
             '1   test@ho | foo\n'
@@ -178,6 +178,7 @@ class TestSimpleAnnotate(TestCaseWithTransport):
 
     def test_annotate_edited_file(self):
         tree = self._setup_edited_file()
+        self.overrideEnv('BZR_EMAIL', 'current@host2')
         out, err = self.run_bzr('annotate file')
         self.assertEqual(
             '1   test@ho | foo\n'
@@ -185,8 +186,24 @@ class TestSimpleAnnotate(TestCaseWithTransport):
             '1   test@ho | gam\n',
             out)
 
+    def test_annotate_edited_file_no_default(self):
+        # Ensure that when no username is available annotate still works.
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
+        # Also, make sure that it's not inferred from mailname.
+        self.overrideAttr(config, '_auto_user_id',
+            lambda: (None, None))
+        tree = self._setup_edited_file()
+        out, err = self.run_bzr('annotate file')
+        self.assertEqual(
+            '1   test@ho | foo\n'
+            '2?  local u | bar\n'
+            '1   test@ho | gam\n',
+            out)
+
     def test_annotate_edited_file_show_ids(self):
         tree = self._setup_edited_file()
+        self.overrideEnv('BZR_EMAIL', 'current@host2')
         out, err = self.run_bzr('annotate file --show-ids')
         self.assertEqual(
             '    rev1 | foo\n'
@@ -211,11 +228,12 @@ class TestSimpleAnnotate(TestCaseWithTransport):
         tree.merge_from_branch(tree.branch, "rev1.1.1")
         # edit the file to be 'resolved' and have a further local edit
         self.build_tree_contents([('file', 'local\nfoo\nbar\nbaz\ngam\n')])
+        return tree
 
     def test_annotated_edited_merged_file_revnos(self):
-        self._create_merged_file()
-        out, err = self.run_bzr('annotate file')
-        email = extract_email_address(Branch.open('.').get_config().username())
+        wt = self._create_merged_file()
+        out, err = self.run_bzr(['annotate', 'file'])
+        email = config.extract_email_address(wt.branch.get_config().username())
         self.assertEqual(
             '3?    %-7s | local\n'
             '1     test@ho | foo\n'
@@ -226,7 +244,7 @@ class TestSimpleAnnotate(TestCaseWithTransport):
 
     def test_annotated_edited_merged_file_ids(self):
         self._create_merged_file()
-        out, err = self.run_bzr('annotate file --show-ids')
+        out, err = self.run_bzr(['annotate', 'file', '--show-ids'])
         self.assertEqual(
             'current: | local\n'
             '    rev1 | foo\n'
@@ -236,45 +254,79 @@ class TestSimpleAnnotate(TestCaseWithTransport):
             out)
 
     def test_annotate_empty_file(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/empty', '')])
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('empty', '')])
         tree.add('empty')
         tree.commit('add empty file')
+        out, err = self.run_bzr(['annotate', 'empty'])
+        self.assertEqual('', out)
 
-        os.chdir('tree')
-        out, err = self.run_bzr('annotate empty')
+    def test_annotate_removed_file(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('empty', '')])
+        tree.add('empty')
+        tree.commit('add empty file')
+        # delete the file.
+        tree.remove('empty')
+        tree.commit('remove empty file')
+        out, err = self.run_bzr(['annotate', '-r1', 'empty'])
         self.assertEqual('', out)
 
     def test_annotate_empty_file_show_ids(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/empty', '')])
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('empty', '')])
         tree.add('empty')
         tree.commit('add empty file')
-
-        os.chdir('tree')
         out, err = self.run_bzr(['annotate', '--show-ids', 'empty'])
         self.assertEqual('', out)
 
     def test_annotate_nonexistant_file(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/file'])
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['file'])
         tree.add(['file'])
         tree.commit('add a file')
-
-        os.chdir('tree')
-        out, err = self.run_bzr("annotate doesnotexist", retcode=3)
+        out, err = self.run_bzr(['annotate', 'doesnotexist'], retcode=3)
         self.assertEqual('', out)
         self.assertEqual("bzr: ERROR: doesnotexist is not versioned.\n", err)
 
     def test_annotate_without_workingtree(self):
-        tree = self.make_branch_and_tree('branch')
-        self.build_tree_contents([('branch/empty', '')])
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('empty', '')])
         tree.add('empty')
         tree.commit('add empty file')
         bzrdir = tree.branch.bzrdir
         bzrdir.destroy_workingtree()
         self.assertFalse(bzrdir.has_workingtree())
-
-        os.chdir('branch')
-        out, err = self.run_bzr('annotate empty')
+        out, err = self.run_bzr(['annotate', 'empty'])
         self.assertEqual('', out)
+
+    def test_annotate_directory(self):
+        """Test --directory option"""
+        wt = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/hello.txt', 'my helicopter\n')])
+        wt.add(['hello.txt'])
+        wt.commit('commit', committer='test@user')
+        out, err = self.run_bzr(['annotate', '-d', 'a', 'hello.txt'])
+        self.assertEqualDiff('1   test@us | my helicopter\n', out)
+
+
+class TestSmartServerAnnotate(tests.TestCaseWithTransport):
+
+    def test_simple_annotate(self):
+        self.setup_smart_server_with_call_log()
+        wt = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/hello.txt', 'my helicopter\n')])
+        wt.add(['hello.txt'])
+        wt.commit('commit', committer='test@user')
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['annotate', "-d", self.get_url('branch'),
+            "hello.txt"])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(16, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.expectFailure("annotate accesses inventories, which require VFS access",
+            self.assertThat, self.hpss_calls, ContainsNoVfsCalls)
