@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2011 Canonical Ltd
+# Copyright (C) 2006-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,34 +22,38 @@ import os
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     revision as _mod_revision,
+    tests,
     )
 from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
-from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests import (
     fixtures,
-    HardlinkFeature,
-    script,
     test_server,
     )
+from bzrlib.tests.features import (
+    HardlinkFeature,
+    )
 from bzrlib.tests.blackbox import test_switch
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.tests.script import run_script
 from bzrlib.urlutils import local_path_to_url, strip_trailing_slash
 from bzrlib.workingtree import WorkingTree
 
 
-class TestBranch(TestCaseWithTransport):
+class TestBranch(tests.TestCaseWithTransport):
 
-    def example_branch(self, path='.'):
-        tree = self.make_branch_and_tree(path)
+    def example_branch(self, path='.', format=None):
+        tree = self.make_branch_and_tree(path, format=format)
         self.build_tree_contents([(path + '/hello', 'foo')])
         tree.add('hello')
         tree.commit(message='setup')
         self.build_tree_contents([(path + '/goodbye', 'baz')])
         tree.add('goodbye')
         tree.commit(message='setup')
+        return tree
 
     def test_branch(self):
         """Branch from one branch to another."""
@@ -61,15 +65,64 @@ class TestBranch(TestCaseWithTransport):
         self.assertFalse(b._transport.has('branch-name'))
         b.bzrdir.open_workingtree().commit(message='foo', allow_pointless=True)
 
+    def test_branch_no_to_location(self):
+        """The to_location is derived from the source branch name."""
+        os.mkdir("something")
+        a = self.example_branch('something/a').branch
+        self.run_bzr('branch something/a')
+        b = branch.Branch.open('a')
+        self.assertEquals(b.last_revision_info(), a.last_revision_info())
+
+    def test_into_colocated(self):
+        """Branch from a branch into a colocated branch."""
+        self.example_branch('a')
+        out, err = self.run_bzr(
+            'init --format=development-colo file:b,branch=orig')
+        self.assertEqual(
+            """Created a lightweight checkout (format: development-colo)\n""",
+            out)
+        self.assertEqual('', err)
+        out, err = self.run_bzr(
+            'branch a file:b,branch=thiswasa')
+        self.assertEqual('', out)
+        self.assertEqual('Branched 2 revisions.\n', err)
+        out, err = self.run_bzr('branches b')
+        self.assertEqual("  orig\n  thiswasa\n", out)
+        self.assertEqual('', err)
+        out,err = self.run_bzr('branch a file:b,branch=orig', retcode=3)
+        self.assertEqual('', out)
+        self.assertEqual(
+            'bzr: ERROR: Already a branch: "file:b,branch=orig".\n', err)
+
+    def test_from_colocated(self):
+        """Branch from a colocated branch into a regular branch."""
+        tree = self.example_branch('a', format='development-colo')
+        tree.bzrdir.create_branch(name='somecolo')
+        out, err = self.run_bzr('branch %s,branch=somecolo' %
+            local_path_to_url('a'))
+        self.assertEqual('', out)
+        self.assertEqual('Branched 0 revisions.\n', err)
+        self.assertPathExists("somecolo")
+
     def test_branch_broken_pack(self):
         """branching with a corrupted pack file."""
         self.example_branch('a')
-        #now add some random corruption
-        fname = 'a/.bzr/repository/packs/' + os.listdir('a/.bzr/repository/packs')[0]
+        # add some corruption
+        packs_dir = 'a/.bzr/repository/packs/'
+        fname = packs_dir + os.listdir(packs_dir)[0]
         with open(fname, 'rb+') as f:
-            f.seek(750)
-            f.write("\xff")
-        self.run_bzr_error(['Corruption while decompressing repository file'], 
+            # Start from the end of the file to avoid choosing a place bigger
+            # than the file itself.
+            f.seek(-5, os.SEEK_END)
+            c = f.read(1)
+            f.seek(-5, os.SEEK_END)
+            # Make sure we inject a value different than the one we just read
+            if c == '\xFF':
+                corrupt = '\x00'
+            else:
+                corrupt = '\xFF'
+            f.write(corrupt) # make sure we corrupt something
+        self.run_bzr_error(['Corruption while decompressing repository file'],
                             'branch a b', retcode=3)
 
     def test_branch_switch_no_branch(self):
@@ -102,21 +155,29 @@ class TestBranch(TestCaseWithTransport):
         #  => new branch will be created, but switch fails and the current
         #     branch is unmodified
         self.example_branch('a')
-        self.make_branch_and_tree('current')
+        tree = self.make_branch_and_tree('current')
+        c1 = tree.commit('some diverged change')
         self.run_bzr_error(['Cannot switch a branch, only a checkout'],
             'branch --switch ../a ../b', working_dir='current')
         a = branch.Branch.open('a')
         b = branch.Branch.open('b')
         self.assertEqual(a.last_revision(), b.last_revision())
         work = branch.Branch.open('current')
-        self.assertEqual(work.last_revision(), _mod_revision.NULL_REVISION)
+        self.assertEqual(work.last_revision(), c1)
+
+    def test_branch_into_empty_dir(self):
+        t = self.example_branch('source')
+        self.make_bzrdir('target')
+        self.run_bzr("branch source target")
+        self.assertEquals(2, len(t.branch.repository.all_revision_ids()))
 
     def test_branch_switch_checkout(self):
         # Checkout in the current directory:
         #  => new branch will be created and checkout bound to the new branch
         self.example_branch('a')
         self.run_bzr('checkout a current')
-        out, err = self.run_bzr('branch --switch ../a ../b', working_dir='current')
+        out, err = self.run_bzr('branch --switch ../a ../b',
+                                working_dir='current')
         a = branch.Branch.open('a')
         b = branch.Branch.open('b')
         self.assertEqual(a.last_revision(), b.last_revision())
@@ -130,7 +191,8 @@ class TestBranch(TestCaseWithTransport):
         #     the new branch
         self.example_branch('a')
         self.run_bzr('checkout --lightweight a current')
-        out, err = self.run_bzr('branch --switch ../a ../b', working_dir='current')
+        out, err = self.run_bzr('branch --switch ../a ../b',
+                                working_dir='current')
         a = branch.Branch.open('a')
         b = branch.Branch.open('b')
         self.assertEqual(a.last_revision(), b.last_revision())
@@ -147,7 +209,7 @@ class TestBranch(TestCaseWithTransport):
 
         def make_shared_tree(path):
             shared_repo.bzrdir.root_transport.mkdir(path)
-            shared_repo.bzrdir.create_branch_convenience('repo/' + path)
+            controldir.ControlDir.create_branch_convenience('repo/' + path)
             return WorkingTree.open('repo/' + path)
         tree_a = make_shared_tree('a')
         self.build_tree(['repo/a/file'])
@@ -287,7 +349,7 @@ class TestBranch(TestCaseWithTransport):
         builder = self.make_branch_builder('source')
         source = fixtures.build_branch_with_non_ancestral_rev(builder)
         source.tags.set_tag('tag-a', 'rev-2')
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         # Now source has a tag not in its ancestry.  Make a branch from it.
         self.run_bzr('branch source new-branch')
         new_branch = branch.Branch.open('new-branch')
@@ -296,16 +358,16 @@ class TestBranch(TestCaseWithTransport):
         new_branch.repository.get_revision('rev-2')
 
 
-class TestBranchStacked(TestCaseWithTransport):
+class TestBranchStacked(tests.TestCaseWithTransport):
     """Tests for branch --stacked"""
 
     def assertRevisionInRepository(self, repo_path, revid):
-        """Check that a revision is in a repository, disregarding stacking."""
+        """Check that a revision is in a repo, disregarding stacking."""
         repo = bzrdir.BzrDir.open(repo_path).open_repository()
         self.assertTrue(repo.has_revision(revid))
 
     def assertRevisionNotInRepository(self, repo_path, revid):
-        """Check that a revision is not in a repository, disregarding stacking."""
+        """Check that a revision is not in a repo, disregarding stacking."""
         repo = bzrdir.BzrDir.open(repo_path).open_repository()
         self.assertFalse(repo.has_revision(revid))
 
@@ -332,7 +394,7 @@ class TestBranchStacked(TestCaseWithTransport):
         # mainline.
         out, err = self.run_bzr(['branch', 'branch', 'newbranch'])
         self.assertEqual('', out)
-        self.assertEqual('Branched 2 revision(s).\n',
+        self.assertEqual('Branched 2 revisions.\n',
             err)
         # it should have preserved the branch format, and so it should be
         # capable of supporting stacking, but not actually have a stacked_on
@@ -383,7 +445,8 @@ class TestBranchStacked(TestCaseWithTransport):
             trunk_tree.branch.base, err)
         self.assertRevisionNotInRepository('newbranch', original_revid)
         new_branch = branch.Branch.open('newbranch')
-        self.assertEqual(trunk_tree.branch.base, new_branch.get_stacked_on_url())
+        self.assertEqual(trunk_tree.branch.base,
+                         new_branch.get_stacked_on_url())
 
     def test_branch_stacked_from_smart_server(self):
         # We can branch stacking on a smart server
@@ -424,7 +487,7 @@ class TestBranchStacked(TestCaseWithTransport):
             err)
 
 
-class TestSmartServerBranching(TestCaseWithTransport):
+class TestSmartServerBranching(tests.TestCaseWithTransport):
 
     def test_branch_from_trivial_branch_to_same_server_branch_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -439,7 +502,10 @@ class TestSmartServerBranching(TestCaseWithTransport):
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(37, self.hpss_calls)
+        self.assertLength(2, self.hpss_connections)
+        self.assertLength(33, self.hpss_calls)
+        self.expectFailure("branching to the same branch requires VFS access",
+            self.assertThat, self.hpss_calls, ContainsNoVfsCalls)
 
     def test_branch_from_trivial_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -454,7 +520,9 @@ class TestSmartServerBranching(TestCaseWithTransport):
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
         self.assertLength(10, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
 
     def test_branch_from_trivial_stacked_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -475,12 +543,14 @@ class TestSmartServerBranching(TestCaseWithTransport):
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(15, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_branch_from_branch_with_tags(self):
         self.setup_smart_server_with_call_log()
         builder = self.make_branch_builder('source')
         source = fixtures.build_branch_with_non_ancestral_rev(builder)
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         source.tags.set_tag('tag-a', 'rev-2')
         source.tags.set_tag('tag-missing', 'missing-rev')
         # Now source has a tag not in its ancestry.  Make a branch from it.
@@ -492,6 +562,8 @@ class TestSmartServerBranching(TestCaseWithTransport):
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(10, self.hpss_calls)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+        self.assertLength(1, self.hpss_connections)
 
     def test_branch_to_stacked_from_trivial_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -510,7 +582,10 @@ class TestSmartServerBranching(TestCaseWithTransport):
         readvs_of_rix_files = [
             c for c in self.hpss_calls
             if c.call.method == 'readv' and c.call.args[-1].endswith('.rix')]
+        self.assertLength(1, self.hpss_connections)
         self.assertLength(0, readvs_of_rix_files)
+        self.expectFailure("branching to stacked requires VFS access",
+            self.assertThat, self.hpss_calls, ContainsNoVfsCalls)
 
 
 class TestRemoteBranch(TestCaseWithSFTPServer):
@@ -537,10 +612,11 @@ class TestRemoteBranch(TestCaseWithSFTPServer):
         self.assertFalse(t.has('remote/file'))
 
 
-class TestDeprecatedAliases(TestCaseWithTransport):
+class TestDeprecatedAliases(tests.TestCaseWithTransport):
 
     def test_deprecated_aliases(self):
-        """bzr branch can be called clone or get, but those names are deprecated.
+        """bzr branch can be called clone or get, but those names are
+        deprecated.
 
         See bug 506265.
         """
@@ -559,7 +635,7 @@ class TestBranchParentLocation(test_switch.TestSwitchParentLocationBase):
                 $ bzr checkout %(option)s repo/trunk checkout
                 $ cd checkout
                 $ bzr branch --switch ../repo/trunk ../repo/branched
-                2>Branched 0 revision(s).
+                2>Branched 0 revisions.
                 2>Tree is up to date at revision 0.
                 2>Switched to branch:...branched...
                 $ cd ..
