@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,15 +19,15 @@
 """Black-box tests for bzr re-sign.
 """
 
-import os
-
-import bzrlib.gpg
+from bzrlib import (
+    gpg,
+    tests,
+    )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.testament import Testament
-from bzrlib.tests import TestCaseInTempDir
 
 
-class ReSign(TestCaseInTempDir):
+class ReSign(tests.TestCaseInTempDir):
 
     def monkey_patch_gpg(self):
         """Monkey patch the gpg signing strategy to be a loopback.
@@ -35,15 +35,8 @@ class ReSign(TestCaseInTempDir):
         This also registers the cleanup, so that we will revert to
         the original gpg strategy when done.
         """
-        self._oldstrategy = bzrlib.gpg.GPGStrategy
-
         # monkey patch gpg signing mechanism
-        bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
-
-        self.addCleanup(self._fix_gpg_strategy)
-
-    def _fix_gpg_strategy(self):
-        bzrlib.gpg.GPGStrategy = self._oldstrategy
+        self.overrideAttr(gpg, 'GPGStrategy', gpg.LoopbackGPGStrategy)
 
     def setup_tree(self):
         wt = BzrDir.create_standalone_workingtree('.')
@@ -93,3 +86,16 @@ class ReSign(TestCaseInTempDir):
         self.assertEqualSignature(repo, 'A')
         self.assertEqualSignature(repo, 'B')
         self.assertEqualSignature(repo, 'C')
+
+    def test_resign_directory(self):
+        """Test --directory option"""
+        wt = BzrDir.create_standalone_workingtree('a')
+        wt.commit("base A", allow_pointless=True, rev_id='A')
+        wt.commit("base B", allow_pointless=True, rev_id='B')
+        wt.commit("base C", allow_pointless=True, rev_id='C')
+        repo = wt.branch.repository
+        self.monkey_patch_gpg()
+        self.run_bzr('re-sign --directory=a -r revid:A')
+        self.assertEqualSignature(repo, 'A')
+        self.run_bzr('re-sign -d a B')
+        self.assertEqualSignature(repo, 'B')
