@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,57 +17,60 @@
 
 """Black-box tests for bzr whoami."""
 
-import os
-
 import bzrlib
 from bzrlib import (
-    osutils,
     config,
+    errors,
+    tests,
     )
-from bzrlib.tests import TestCaseWithTransport
 
 
-class TestWhoami(TestCaseWithTransport):
+class TestWhoami(tests.TestCaseWithTransport):
 
-    def test_whoami(self):
+    def assertWhoAmI(self, expected, *cmd_args, **kwargs):
+        out, err = self.run_bzr(('whoami',) + cmd_args, **kwargs)
+        self.assertEquals('', err)
+        lines = out.splitlines()
+        self.assertLength(1, lines)
+        self.assertEquals(expected, lines[0].rstrip())
+
+    def test_whoami_no_args_no_conf(self):
         # this should always identify something, if only "john@localhost"
         out = self.run_bzr("whoami")[0]
         self.assertTrue(len(out) > 0)
         self.assertEquals(1, out.count('@'))
 
+    def test_whoami_email_no_args(self):
         out = self.run_bzr("whoami --email")[0]
         self.assertTrue(len(out) > 0)
         self.assertEquals(1, out.count('@'))
 
+    def test_whoami_email_arg(self):
+        # whoami --email is mutually exclusive with any arguments
+        out = self.run_bzr("whoami --email 'foo <foo@example.com>'", 3)[0]
+        self.assertEquals("", out)
+
     def test_whoami_branch(self):
         """branch specific user identity works."""
         wt = self.make_branch_and_tree('.')
         b = bzrlib.branch.Branch.open('.')
-        b.get_config().set_user_option('email',
-                                       'Branch Identity <branch@identi.ty>')
-        whoami = self.run_bzr("whoami")[0]
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email")[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        b.get_config_stack().set('email', 'Branch Identity <branch@identi.ty>')
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>')
+        self.assertWhoAmI('branch@identi.ty', '--email')
 
         # Verify that the environment variable overrides the value
         # in the file
         self.overrideEnv('BZR_EMAIL', 'Different ID <other@environ.ment>')
-        whoami = self.run_bzr("whoami")[0]
-        self.assertEquals('Different ID <other@environ.ment>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email")[0]
-        self.assertEquals('other@environ.ment\n', whoami_email)
+        self.assertWhoAmI('Different ID <other@environ.ment>')
+        self.assertWhoAmI('other@environ.ment', '--email')
 
     def test_whoami_utf8(self):
         """verify that an identity can be in utf-8."""
-        wt = self.make_branch_and_tree('.')
         self.run_bzr(['whoami', u'Branch Identity \u20ac <branch@identi.ty>'],
                      encoding='utf-8')
-        whoami = self.run_bzr("whoami", encoding='utf-8')[0]
-        self.assertEquals('Branch Identity \xe2\x82\xac ' +
-                          '<branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email", encoding='utf-8')[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        self.assertWhoAmI('Branch Identity \xe2\x82\xac <branch@identi.ty>',
+                          encoding='utf-8')
+        self.assertWhoAmI('branch@identi.ty', '--email')
 
     def test_whoami_ascii(self):
         """
@@ -76,12 +79,12 @@ class TestWhoami(TestCaseWithTransport):
         """
         wt = self.make_branch_and_tree('.')
         b = bzrlib.branch.Branch.open('.')
-        b.get_config().set_user_option('email', u'Branch Identity \u20ac ' +
-                                       '<branch@identi.ty>')
-        whoami = self.run_bzr("whoami", encoding='ascii')[0]
-        self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email", encoding='ascii')[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        b.get_config_stack().set(
+            'email', u'Branch Identity \u20ac ' + '<branch@identi.ty>')
+        self.assertWhoAmI('Branch Identity ? <branch@identi.ty>',
+                          encoding='ascii')
+        self.assertWhoAmI('branch@identi.ty', '--email',
+                          encoding='ascii')
 
     def test_warning(self):
         """verify that a warning is displayed if no email is given."""
@@ -105,32 +108,39 @@ class TestWhoami(TestCaseWithTransport):
     def test_whoami_directory(self):
         """Test --directory option."""
         wt = self.make_branch_and_tree('subdir')
-        c = wt.branch.get_config()
-        c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
-        out, err = self.run_bzr("whoami --directory subdir")
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        c = wt.branch.get_config_stack()
+        c.set('email', 'Branch Identity <branch@identi.ty>')
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>',
+                          '--directory', 'subdir')
         self.run_bzr(['whoami', '--directory', 'subdir', '--branch',
                       'Changed Identity <changed@identi.ty>'])
+        c = wt.branch.get_config_stack()
         self.assertEquals('Changed Identity <changed@identi.ty>',
-                          c.get_user_option('email'))
+                          c.get('email'))
 
     def test_whoami_remote_directory(self):
         """Test --directory option with a remote directory."""
         wt = self.make_branch_and_tree('subdir')
-        c = wt.branch.get_config()
-        c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
+        c = wt.branch.get_config_stack()
+        c.set('email', 'Branch Identity <branch@identi.ty>')
         url = self.get_readonly_url() + '/subdir'
-        out, err = self.run_bzr(['whoami', '--directory', url])
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>',
+                          '--directory', url)
         url = self.get_url('subdir')
         self.run_bzr(['whoami', '--directory', url, '--branch',
                       'Changed Identity <changed@identi.ty>'])
         # The identity has been set in the branch config (but not the global
         # config)
+        c = wt.branch.get_config_stack()
         self.assertEquals('Changed Identity <changed@identi.ty>',
-                          c.get_user_option('email'))
-        global_conf = config.GlobalConfig()
-        self.assertEquals(None, global_conf.get_user_option('email'))
+                          c.get('email'))
+        # Ensuring that the value does not come from the bazaar.conf file
+        # itself requires some isolation setup
+        self.overrideEnv('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideAttr(config, '_auto_user_id', lambda: (None, None))
+        global_conf = config.GlobalStack()
+        self.assertRaises(errors.NoWhoami, global_conf.get, 'email')
 
     def test_whoami_nonbranch_directory(self):
         """Test --directory mentioning a non-branch directory."""
