@@ -21,7 +21,6 @@ import os
 from bzrlib import (
     config,
     errors,
-    graph,
     inventory,
     osutils,
     repository,
@@ -29,6 +28,9 @@ from bzrlib import (
     tests,
     )
 from bzrlib.tests import per_repository
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestCommitBuilder(per_repository.TestCaseWithRepository):
@@ -37,7 +39,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         branch = self.make_branch('.')
         branch.repository.lock_write()
         builder = branch.repository.get_commit_builder(
-            branch, [], branch.get_config())
+            branch, [], branch.get_config_stack())
         self.assertIsInstance(builder, repository.CommitBuilder)
         self.assertTrue(builder.random_revid)
         branch.repository.commit_write_group()
@@ -61,6 +63,9 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
             repo = tree.branch.repository
             self.record_root(builder, tree)
             builder.finish_inventory()
@@ -85,11 +90,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         finally:
             tree.unlock()
 
-    def test_abort(self):
+    def test_abort_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
             self.record_root(builder, tree)
             builder.finish_inventory()
             builder.abort()
@@ -112,12 +120,13 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         finally:
             tree.unlock()
 
-    def test_commit_message(self):
+    def test_commit_lossy(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
-            builder = tree.branch.get_commit_builder([])
-            self.record_root(builder, tree)
+            builder = tree.branch.get_commit_builder([], lossy=True)
+            list(builder.record_iter_changes(tree, tree.last_revision(),
+                tree.iter_changes(tree.basis_tree())))
             builder.finish_inventory()
             rev_id = builder.commit('foo bar blah')
         finally:
@@ -125,7 +134,36 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         rev = tree.branch.repository.get_revision(rev_id)
         self.assertEqual('foo bar blah', rev.message)
 
-    def test_commit_with_revision_id(self):
+    def test_commit_message(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        try:
+            builder = tree.branch.get_commit_builder([])
+            list(builder.record_iter_changes(tree, tree.last_revision(),
+                tree.iter_changes(tree.basis_tree())))
+            builder.finish_inventory()
+            rev_id = builder.commit('foo bar blah')
+        finally:
+            tree.unlock()
+        rev = tree.branch.repository.get_revision(rev_id)
+        self.assertEqual('foo bar blah', rev.message)
+
+    def test_updates_branch(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        try:
+            builder = tree.branch.get_commit_builder([])
+            list(builder.record_iter_changes(tree, tree.last_revision(),
+                tree.iter_changes(tree.basis_tree())))
+            builder.finish_inventory()
+            will_update_branch = builder.updates_branch
+            rev_id = builder.commit('might update the branch')
+        finally:
+            tree.unlock()
+        actually_updated_branch = (tree.branch.last_revision() == rev_id)
+        self.assertEquals(actually_updated_branch, will_update_branch)
+
+    def test_commit_with_revision_id_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
@@ -143,6 +181,9 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             except errors.CannotSetRevisionId:
                 # This format doesn't support supplied revision ids
                 return
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
             self.assertFalse(builder.random_revid)
             self.record_root(builder, tree)
             builder.finish_inventory()
@@ -192,7 +233,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         # on deserialisation, but thats all the current contract guarantees
         # anyway.
         self.assertEqual(revision_id,
-            tree.branch.repository.get_inventory(revision_id).revision_id)
+            tree.branch.repository.revision_tree(revision_id).get_revision_id())
 
     def test_commit_without_root_errors(self):
         tree = self.make_branch_and_tree(".")
@@ -207,6 +248,8 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
                 except:
                     builder.abort()
                     raise
+                else:
+                    builder.commit("msg")
             self.assertRaises(errors.RootMissing, do_commit)
         finally:
             tree.unlock()
@@ -217,8 +260,11 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         try:
             self.build_tree(['foo'])
             tree.add('foo', 'foo-id')
-            entry = tree.inventory['foo-id']
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
+            entry = tree.inventory['foo-id']
             self.assertRaises(errors.RootMissing,
                 builder.record_entry_contents, entry, [], 'foo', tree,
                     tree.path_content_summary('foo'))
@@ -226,7 +272,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         finally:
             tree.unlock()
 
-    def test_commit_unchanged_root(self):
+    def test_commit_unchanged_root_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         old_revision_id = tree.commit('')
         tree.lock_write()
@@ -235,6 +281,10 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         self.addCleanup(parent_tree.unlock)
         builder = tree.branch.get_commit_builder([old_revision_id])
         try:
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
+            builder.will_record_deletes()
             ie = inventory.make_entry('directory', '', None,
                     tree.get_root_id())
             delta, version_recorded, fs_hash = builder.record_entry_contents(
@@ -250,7 +300,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             if got_new_revision:
                 self.assertEqual(('', '', ie.file_id, ie), delta)
                 # The delta should be tracked
-                self.assertEqual(delta, builder._basis_delta[-1])
+                self.assertEqual(delta, builder.get_basis_delta()[-1])
             else:
                 self.assertEqual(None, delta)
             # Directories do not get hashed.
@@ -274,23 +324,27 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             # pointless commit.
             self.assertFalse(builder.any_changes())
             builder.finish_inventory()
-            new_root = tree.branch.repository.get_inventory(
-                builder._new_revision_id).root
+            builder.commit('')
+            builder_tree = builder.revision_tree()
+            new_root_id = builder_tree.get_root_id()
+            new_root_revision = builder_tree.get_file_revision(new_root_id)
             if tree.branch.repository.supports_rich_root():
                 # We should not have seen a new root revision
-                self.assertEqual(old_revision_id, new_root.revision)
+                self.assertEqual(old_revision_id, new_root_revision)
             else:
                 # We should see a new root revision
-                self.assertNotEqual(old_revision_id, new_root.revision)
+                self.assertNotEqual(old_revision_id, new_root_revision)
         finally:
-            builder.abort()
             tree.unlock()
 
-    def test_commit(self):
+    def test_commit_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't "
+                    "support record_entry_contents")
             self.record_root(builder, tree)
             builder.finish_inventory()
             rev_id = builder.commit('foo bar')
@@ -318,6 +372,9 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             builder = tree.branch.get_commit_builder([old_revision_id])
             total_delta = []
             try:
+                if not builder.supports_record_entry_contents:
+                    raise tests.TestNotApplicable("CommitBuilder doesn't "
+                        "support record_entry_contents")
                 parent_invs = [basis.inventory]
                 builder.will_record_deletes()
                 if builder.record_root_entry:
@@ -373,6 +430,9 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             basis = tree.branch.repository.revision_tree(rev_id)
             builder = tree.branch.get_commit_builder([rev_id])
             try:
+                if not builder.supports_record_entry_contents:
+                    raise tests.TestNotApplicable("CommitBuilder doesn't "
+                        "support record_entry_contents")
                 builder.will_record_deletes()
                 if builder.record_root_entry is True:
                     parent_invs = [basis.inventory]
@@ -382,7 +442,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
                 # the delta should be returned, and recorded in _basis_delta
                 delta = builder.record_delete("foo", "foo-id")
                 self.assertEqual(("foo", None, "foo-id", None), delta)
-                self.assertEqual(delta, builder._basis_delta[-1])
+                self.assertEqual(delta, builder.get_basis_delta()[-1])
                 builder.finish_inventory()
                 rev_id2 = builder.commit('delete foo')
             except:
@@ -404,13 +464,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         try:
             builder = tree.branch.get_commit_builder([rev_id])
             try:
+                builder.will_record_deletes()
                 delete_change = ('foo-id', ('foo', None), True, (True, False),
                     (tree.path2id(''), None), ('foo', None), ('file', None),
                     (False, None))
                 list(builder.record_iter_changes(tree, rev_id,
                     [delete_change]))
                 self.assertEqual(("foo", None, "foo-id", None),
-                    builder._basis_delta[0])
+                    builder.get_basis_delta()[0])
                 self.assertTrue(builder.any_changes())
                 builder.finish_inventory()
                 rev_id2 = builder.commit('delete foo')
@@ -433,6 +494,9 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         try:
             builder = tree.branch.get_commit_builder([rev_id])
             try:
+                if not builder.supports_record_entry_contents:
+                    raise tests.TestNotApplicable("CommitBuilder doesn't "
+                        "support record_entry_contents")
                 self.record_root(builder, tree)
                 self.assertRaises(AssertionError,
                     builder.record_delete, "foo", "foo-id")
@@ -441,11 +505,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         finally:
             tree.unlock()
 
-    def test_revision_tree(self):
+    def test_revision_tree_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't "
+                    "support record_entry_contents")
             self.record_root(builder, tree)
             builder.finish_inventory()
             rev_id = builder.commit('foo bar')
@@ -475,7 +542,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             # Just a couple simple tests to ensure that it actually follows
             # the RevisionTree api.
             self.assertEqual(rev_id, rev_tree.get_revision_id())
-            self.assertEqual([], rev_tree.get_parent_ids())
+            self.assertEqual((), tuple(rev_tree.get_parent_ids()))
         finally:
             tree.unlock()
 
@@ -490,7 +557,8 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         basis_tree = tree.basis_tree()
         basis_tree.lock_read()
         self.addCleanup(basis_tree.unlock)
-        self.assertEqual(rev_id, basis_tree.inventory.root.revision)
+        self.assertEqual(rev_id,
+            basis_tree.get_file_revision(basis_tree.get_root_id()))
 
     def _get_revtrees(self, tree, revision_ids):
         tree.lock_read()
@@ -510,11 +578,13 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         rev1 = tree.commit('')
         rev2 = tree.commit('')
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.inventory.root.revision)
+        self.assertEqual(rev1, tree1.get_file_revision(tree1.get_root_id()))
         if tree.branch.repository.supports_rich_root():
-            self.assertEqual(rev1, tree2.inventory.root.revision)
+            self.assertEqual(rev1,
+                tree2.get_file_revision(tree2.get_root_id()))
         else:
-            self.assertEqual(rev2, tree2.inventory.root.revision)
+            self.assertEqual(rev2,
+                tree2.get_file_revision(tree2.get_root_id()))
 
     def _add_commit_check_unchanged(self, tree, name, mini_commit=None):
         tree.add([name], [name + 'id'])
@@ -527,8 +597,8 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             mini_commit = self.mini_commit
         rev2 = mini_commit(tree, name, name, False, False)
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.inventory[file_id].revision)
-        self.assertEqual(rev1, tree2.inventory[file_id].revision)
+        self.assertEqual(rev1, tree1.get_file_revision(file_id))
+        self.assertEqual(rev1, tree2.get_file_revision(file_id))
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
         self.assertFileGraph(expected_graph, tree, (file_id, rev1))
@@ -557,8 +627,8 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         tree.add(['dir/content'], ['contentid'])
         rev2 = tree.commit('')
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.inventory['dirid'].revision)
-        self.assertEqual(rev1, tree2.inventory['dirid'].revision)
+        self.assertEqual(rev1, tree1.get_file_revision('dirid'))
+        self.assertEqual(rev1, tree2.get_file_revision('dirid'))
         file_id = 'dirid'
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
@@ -579,14 +649,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
 
     def test_last_modified_revision_after_commit_link_unchanged(self):
         # committing without changing a link does not change the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_check_unchanged(tree, 'link')
 
     def test_last_modified_revision_after_commit_link_unchanged_ric(self):
         # committing without changing a link does not change the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_check_unchanged(tree, 'link',
@@ -661,14 +731,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
 
     def test_last_modified_revision_after_rename_link_changes(self):
         # renaming a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_renamed_check_changed(tree, 'link')
 
     def test_last_modified_revision_after_rename_link_changes_ric(self):
         # renaming a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_renamed_check_changed(tree, 'link',
@@ -736,14 +806,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
 
     def test_last_modified_revision_after_reparent_link_changes(self):
         # reparenting a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_reparent_check_changed(tree, 'link')
 
     def test_last_modified_revision_after_reparent_link_changes_ric(self):
         # reparenting a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_reparent_check_changed(tree, 'link',
@@ -767,8 +837,8 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         rev2 = mini_commit(tree, name, tree.id2path(file_id),
             expect_fs_hash=expect_fs_hash)
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.inventory[file_id].revision)
-        self.assertEqual(rev2, tree2.inventory[file_id].revision)
+        self.assertEqual(rev1, tree1.get_file_revision(file_id))
+        self.assertEqual(rev2, tree2.get_file_revision(file_id))
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
         expected_graph[(file_id, rev2)] = ((file_id, rev1),)
@@ -794,61 +864,70 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             # record_entry_contents.
             parent_ids = tree.get_parent_ids()
             builder = tree.branch.get_commit_builder(parent_ids)
-            parent_tree = tree.basis_tree()
-            parent_tree.lock_read()
-            self.addCleanup(parent_tree.unlock)
-            parent_invs = [parent_tree.inventory]
-            for parent_id in parent_ids[1:]:
-                parent_invs.append(tree.branch.repository.revision_tree(
-                    parent_id).inventory)
-            # root
-            builder.record_entry_contents(
-                inventory.make_entry('directory', '', None,
-                    tree.get_root_id()), parent_invs, '', tree,
-                    tree.path_content_summary(''))
-            def commit_id(file_id):
-                old_ie = tree.inventory[file_id]
-                path = tree.id2path(file_id)
-                ie = inventory.make_entry(tree.kind(file_id), old_ie.name,
-                    old_ie.parent_id, file_id)
-                content_summary = tree.path_content_summary(path)
-                if content_summary[0] == 'tree-reference':
-                    content_summary = content_summary[:3] + (
-                        tree.get_reference_revision(file_id),)
-                return builder.record_entry_contents(ie, parent_invs, path,
-                    tree, content_summary)
+            try:
+                if not builder.supports_record_entry_contents:
+                    raise tests.TestNotApplicable("CommitBuilder doesn't "
+                        "support record_entry_contents")
+                builder.will_record_deletes()
+                parent_tree = tree.basis_tree()
+                parent_tree.lock_read()
+                self.addCleanup(parent_tree.unlock)
+                parent_invs = [parent_tree.inventory]
+                for parent_id in parent_ids[1:]:
+                    parent_invs.append(tree.branch.repository.revision_tree(
+                        parent_id).inventory)
+                # root
+                builder.record_entry_contents(
+                    inventory.make_entry('directory', '', None,
+                        tree.get_root_id()), parent_invs, '', tree,
+                        tree.path_content_summary(''))
+                def commit_id(file_id):
+                    old_ie = tree.inventory[file_id]
+                    path = tree.id2path(file_id)
+                    ie = inventory.make_entry(tree.kind(file_id), old_ie.name,
+                        old_ie.parent_id, file_id)
+                    content_summary = tree.path_content_summary(path)
+                    if content_summary[0] == 'tree-reference':
+                        content_summary = content_summary[:3] + (
+                            tree.get_reference_revision(file_id),)
+                    return builder.record_entry_contents(ie, parent_invs, path,
+                        tree, content_summary)
 
-            file_id = tree.path2id(new_name)
-            parent_id = tree.inventory[file_id].parent_id
-            if parent_id != tree.get_root_id():
-                commit_id(parent_id)
-            # because a change of some sort is meant to have occurred,
-            # recording the entry must return True.
-            delta, version_recorded, fs_hash = commit_id(file_id)
-            if records_version:
-                self.assertTrue(version_recorded)
+                file_id = tree.path2id(new_name)
+                parent_id = tree.inventory[file_id].parent_id
+                if parent_id != tree.get_root_id():
+                    commit_id(parent_id)
+                # because a change of some sort is meant to have occurred,
+                # recording the entry must return True.
+                delta, version_recorded, fs_hash = commit_id(file_id)
+                if records_version:
+                    self.assertTrue(version_recorded)
+                else:
+                    self.assertFalse(version_recorded)
+                if expect_fs_hash:
+                    tree_file_stat = tree.get_file_with_stat(file_id)
+                    tree_file_stat[0].close()
+                    self.assertEqual(2, len(fs_hash))
+                    self.assertEqual(tree.get_file_sha1(file_id), fs_hash[0])
+                    self.assertEqualStat(tree_file_stat[1], fs_hash[1])
+                else:
+                    self.assertEqual(None, fs_hash)
+                new_entry = builder.new_inventory[file_id]
+                if delta_against_basis:
+                    expected_delta = (name, new_name, file_id, new_entry)
+                    # The delta should be recorded
+                    self.assertEqual(expected_delta,
+                        builder.get_basis_delta()[-1])
+                else:
+                    expected_delta = None
+                self.assertEqual(expected_delta, delta)
+                builder.finish_inventory()
+            except:
+                builder.abort()
+                raise
             else:
-                self.assertFalse(version_recorded)
-            if expect_fs_hash:
-                tree_file_stat = tree.get_file_with_stat(file_id)
-                tree_file_stat[0].close()
-                self.assertEqual(2, len(fs_hash))
-                self.assertEqual(tree.get_file_sha1(file_id), fs_hash[0])
-                self.assertEqualStat(tree_file_stat[1], fs_hash[1])
-            else:
-                self.assertEqual(None, fs_hash)
-            new_entry = builder.new_inventory[file_id]
-            if delta_against_basis:
-                expected_delta = (name, new_name, file_id, new_entry)
-                # The delta should be recorded
-                self.assertEqual(expected_delta, builder._basis_delta[-1])
-            else:
-                expected_delta = None
-            self.assertEqual(expected_delta, delta)
-            builder.finish_inventory()
-            rev2 = builder.commit('')
+                rev2 = builder.commit('')
         except:
-            builder.abort()
             tree.unlock()
             raise
         try:
@@ -879,13 +958,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             # record_entry_contents.
             parent_ids = tree.get_parent_ids()
             builder = tree.branch.get_commit_builder(parent_ids)
+            builder.will_record_deletes()
             parent_tree = tree.basis_tree()
             parent_tree.lock_read()
             self.addCleanup(parent_tree.unlock)
-            parent_invs = [parent_tree.inventory]
+            parent_trees = [parent_tree]
             for parent_id in parent_ids[1:]:
-                parent_invs.append(tree.branch.repository.revision_tree(
-                    parent_id).inventory)
+                parent_trees.append(tree.branch.repository.revision_tree(
+                    parent_id))
             changes = list(tree.iter_changes(parent_tree))
             result = list(builder.record_iter_changes(tree, parent_ids[0],
                 changes))
@@ -900,22 +980,25 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
                 self.assertEqualStat(result[2][1], tree_file_stat[1])
             else:
                 self.assertEqual([], result)
-            delta = builder._basis_delta
+            self.assertIs(None, builder.new_inventory)
+            builder.finish_inventory()
+            if tree.branch.repository._format.supports_full_versioned_files:
+                inv_key = (builder._new_revision_id,)
+                inv_sha1 = tree.branch.repository.inventories.get_sha1s(
+                                [inv_key])[inv_key]
+                self.assertEqual(inv_sha1, builder.inv_sha1)
+            self.assertIs(None, builder.new_inventory)
+            rev2 = builder.commit('')
+            delta = builder.get_basis_delta()
             delta_dict = dict((change[2], change) for change in delta)
             version_recorded = (file_id in delta_dict and
                 delta_dict[file_id][3] is not None and
-                delta_dict[file_id][3].revision == builder._new_revision_id)
+                delta_dict[file_id][3].revision == rev2)
             if records_version:
                 self.assertTrue(version_recorded)
             else:
                 self.assertFalse(version_recorded)
-            self.assertIs(None, builder.new_inventory)
-            builder.finish_inventory()
-            inv_key = (builder._new_revision_id,)
-            inv_sha1 = tree.branch.repository.inventories.get_sha1s(
-                            [inv_key])[inv_key]
-            self.assertEqual(inv_sha1, builder.inv_sha1)
-            self.assertIs(None, builder.new_inventory)
+
             new_inventory = builder.revision_tree().inventory
             new_entry = new_inventory[file_id]
             if delta_against_basis:
@@ -924,7 +1007,6 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             else:
                 expected_delta = None
                 self.assertFalse(version_recorded)
-            rev2 = builder.commit('')
             tree.set_parent_ids([rev2])
         except:
             builder.abort()
@@ -939,7 +1021,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         # (closest to a public per-file graph API we have today)
         tree.lock_read()
         self.addCleanup(tree.unlock)
-        g = dict(graph.Graph(tree.branch.repository.texts).iter_ancestry([tip]))
+        g = dict(tree.branch.repository.get_file_graph().iter_ancestry([tip]))
         self.assertEqual(expected_graph, g)
 
     def test_last_modified_revision_after_content_file_changes(self):
@@ -963,7 +1045,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
 
     def test_last_modified_revision_after_content_link_changes(self):
         # changing a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         def change_link():
@@ -976,7 +1058,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         if file_id is None:
             file_id = link
         # changing a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink(target, link)
         def change_link():
@@ -992,7 +1074,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             'link', 'target', 'newtarget')
 
     def test_last_modified_rev_after_content_unicode_link_changes_ric(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_last_mod_rev_after_content_link_changes_ric(
             u'li\u1234nk', u'targ\N{Euro Sign}t', u'n\N{Euro Sign}wtarget',
 
@@ -1020,7 +1102,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         rev4 = mini_commit(tree1, 'new_' + name, 'new_' + name,
             expect_fs_hash=expect_fs_hash)
         tree3, = self._get_revtrees(tree1, [rev4])
-        self.assertEqual(rev4, tree3.inventory[name + 'id'].revision)
+        self.assertEqual(rev4, tree3.get_file_revision(name + 'id'))
         file_id = name + 'id'
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
@@ -1057,14 +1139,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
 
     def test_last_modified_revision_after_merge_link_changes(self):
         # merge a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge(tree1, 'link')
 
     def test_last_modified_revision_after_merge_link_changes_ric(self):
         # merge a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge(tree1, 'link',
@@ -1086,7 +1168,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             rev3 = mini_commit(in_tree, name, 'new_' + name, False,
                 delta_against_basis=changed_in_tree)
             tree3, = self._get_revtrees(in_tree, [rev2])
-            self.assertEqual(rev2, tree3.inventory[name + 'id'].revision)
+            self.assertEqual(rev2, tree3.get_file_revision(name + 'id'))
             file_id = name + 'id'
             expected_graph = {}
             expected_graph[(file_id, rev1)] = ()
@@ -1118,7 +1200,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         rev3 = mini_commit(tree1, None, 'name', False)
         tree3, = self._get_revtrees(tree1, [rev2])
         # in rev2, name should be only changed in rev2
-        self.assertEqual(rev2, tree3.inventory[file_id].revision)
+        self.assertEqual(rev2, tree3.get_file_revision(file_id))
         expected_graph = {}
         expected_graph[(file_id, rev2)] = ()
         self.assertFileGraph(expected_graph, tree1, (file_id, rev2))
@@ -1151,14 +1233,14 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
 
     def test_last_modified_revision_after_converged_merge_link_unchanged(self):
         # merge a link that changed preserves the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge_converged(tree1, 'link')
 
     def test_last_modified_revision_after_converged_merge_link_unchanged_ric(self):
         # merge a link that changed preserves the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge_converged(tree1, 'link',
@@ -1204,7 +1286,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         self.build_tree([name])
 
     def make_link(self, name):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         os.symlink('target', name)
 
     def make_reference(self, name):
@@ -1219,7 +1301,10 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         make_before(path)
 
         def change_kind():
-            osutils.delete_any(path)
+            if osutils.file_kind(path) == "directory":
+                osutils.rmtree(path)
+            else:
+                osutils.delete_any(path)
             make_after(path)
 
         self._add_commit_change_check_changed(tree, path, change_kind,
@@ -1230,16 +1315,26 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
             expect_fs_hash=True)
 
     def test_last_modified_dir_file_ric(self):
-        self._check_kind_change(self.make_dir, self.make_file,
-            expect_fs_hash=True,
-            mini_commit=self.mini_commit_record_iter_changes)
+        try:
+            self._check_kind_change(self.make_dir, self.make_file,
+                expect_fs_hash=True,
+                mini_commit=self.mini_commit_record_iter_changes)
+        except errors.UnsupportedKindChange:
+            raise tests.TestSkipped(
+                "tree does not support changing entry kind from "
+                "directory to file")
 
     def test_last_modified_dir_link(self):
         self._check_kind_change(self.make_dir, self.make_link)
 
     def test_last_modified_dir_link_ric(self):
-        self._check_kind_change(self.make_dir, self.make_link,
-            mini_commit=self.mini_commit_record_iter_changes)
+        try:
+            self._check_kind_change(self.make_dir, self.make_link,
+                mini_commit=self.mini_commit_record_iter_changes)
+        except errors.UnsupportedKindChange:
+            raise tests.TestSkipped(
+                "tree does not support changing entry kind from "
+                "directory to link")
 
     def test_last_modified_link_file(self):
         self._check_kind_change(self.make_link, self.make_file,
@@ -1276,7 +1371,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         branch.repository.lock_write()
         self.addCleanup(branch.repository.unlock)
         self.assertRaises(ValueError, branch.repository.get_commit_builder,
-            branch, [], branch.get_config(),
+            branch, [], branch.get_config_stack(),
             revprops={'invalid': u'property\rwith\r\ninvalid chars'})
 
     def test_commit_builder_commit_with_invalid_message(self):
@@ -1284,7 +1379,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         branch.repository.lock_write()
         self.addCleanup(branch.repository.unlock)
         builder = branch.repository.get_commit_builder(branch, [],
-            branch.get_config())
+            branch.get_config_stack())
         self.addCleanup(branch.repository.abort_write_group)
         self.assertRaises(ValueError, builder.commit,
             u'Invalid\r\ncommit message\r\n')
@@ -1296,7 +1391,7 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         self.addCleanup(branch.repository.unlock)
         self.assertRaises(UnicodeDecodeError,
             branch.repository.get_commit_builder,
-            branch, [], branch.get_config(),
+            branch, [], branch.get_config_stack(),
             committer="Erik B\xe5gfors <erik@example.com>")
 
     def test_stacked_repositories_reject_commit_builder(self):
@@ -1313,10 +1408,10 @@ class TestCommitBuilder(per_repository.TestCaseWithRepository):
         self.addCleanup(repo_local.lock_write().unlock)
         if not repo_local._format.supports_chks:
             self.assertRaises(errors.BzrError, repo_local.get_commit_builder,
-                branch, [], branch.get_config())
+                branch, [], branch.get_config_stack())
         else:
             builder = repo_local.get_commit_builder(branch, [],
-                                                    branch.get_config())
+                                                    branch.get_config_stack())
             builder.abort()
 
     def test_committer_no_username(self):
