@@ -15,7 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 import os
 
 from bzrlib import (
@@ -26,18 +25,10 @@ from bzrlib import (
     mutabletree,
     osutils,
     revision as _mod_revision,
+    tests,
     ui,
-    uncommit,
-    workingtree,
     )
-from bzrlib.errors import (NotBranchError, NotVersionedError,
-                           UnsupportedOperation)
-from bzrlib.osutils import pathjoin, getcwd
-from bzrlib.tests import TestCase
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
-from bzrlib.trace import mutter
-from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
-                                WorkingTree)
 from bzrlib.tests.testui import ProgressRecordingUIFactory
 
 
@@ -156,8 +147,12 @@ class TestCommit(TestCaseWithWorkingTree):
         wt2 = wt.bzrdir.sprout('to').open_workingtree()
         wt2.commit('change_right')
         wt.merge_from_branch(wt2.branch)
-        self.assertRaises(errors.CannotCommitSelectedFileMerge,
-            wt.commit, 'test', exclude=['foo'])
+        try:
+            self.assertRaises(errors.CannotCommitSelectedFileMerge,
+                wt.commit, 'test', exclude=['foo'])
+        except errors.ExcludesUnsupported:
+            raise tests.TestNotApplicable("excludes not supported by this "
+                "repository format")
 
     def test_commit_exclude_exclude_changed_is_pointless(self):
         tree = self.make_branch_and_tree('.')
@@ -165,14 +160,22 @@ class TestCommit(TestCaseWithWorkingTree):
         tree.smart_add(['.'])
         tree.commit('setup test')
         self.build_tree_contents([('a', 'new contents for "a"\n')])
-        self.assertRaises(errors.PointlessCommit, tree.commit, 'test',
-            exclude=['a'], allow_pointless=False)
+        try:
+            self.assertRaises(errors.PointlessCommit, tree.commit, 'test',
+                exclude=['a'], allow_pointless=False)
+        except errors.ExcludesUnsupported:
+            raise tests.TestNotApplicable("excludes not supported by this "
+                "repository format")
 
     def test_commit_exclude_excludes_modified_files(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b', 'c'])
         tree.smart_add(['.'])
-        tree.commit('test', exclude=['b', 'c'])
+        try:
+            tree.commit('test', exclude=['b', 'c'])
+        except errors.ExcludesUnsupported:
+            raise tests.TestNotApplicable("excludes not supported by this "
+                "repository format")
         # If b was excluded it will still be 'added' in status.
         tree.lock_read()
         self.addCleanup(tree.unlock)
@@ -185,7 +188,11 @@ class TestCommit(TestCaseWithWorkingTree):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b'])
         tree.smart_add(['.'])
-        tree.commit('test', specific_files=['a'], exclude=['a/b'])
+        try:
+            tree.commit('test', specific_files=['a'], exclude=['a/b'])
+        except errors.ExcludesUnsupported:
+            raise tests.TestNotApplicable("excludes not supported by this "
+                "repository format")
         # If a/b was excluded it will still be 'added' in status.
         tree.lock_read()
         self.addCleanup(tree.unlock)
@@ -545,7 +552,7 @@ class TestCommitProgress(TestCaseWithWorkingTree):
         """Make sure a start commit hook can modify the tree that is
         committed."""
         def start_commit_hook_adds_file(tree):
-            open(tree.abspath("newfile"), 'w').write("data")
+            with open(tree.abspath("newfile"), 'w') as f: f.write("data")
             tree.add(["newfile"])
         def restoreDefaults():
             mutabletree.MutableTree.hooks['start_commit'] = []
@@ -566,7 +573,7 @@ class TestCommitProgress(TestCaseWithWorkingTree):
                 mutabletree.PostCommitHookParams))
             self.assertTrue(isinstance(params.mutable_tree,
                 mutabletree.MutableTree))
-            open(tree.abspath("newfile"), 'w').write("data")
+            with open(tree.abspath("newfile"), 'w') as f: f.write("data")
             params.mutable_tree.add(["newfile"])
         tree = self.make_branch_and_tree('.')
         mutabletree.MutableTree.hooks.install_named_hook(
