@@ -15,9 +15,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 
 from bzrlib import tests
+from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import per_workingtree
 
 
@@ -31,6 +31,12 @@ class TestPull(per_workingtree.TestCaseWithWorkingTree):
         tree_b = self.make_branch_and_tree('to')
         return tree, tree_b
 
+    def test_pull_null(self):
+        tree_a, tree_b = self.get_pullable_trees()
+        root_id = tree_a.get_root_id()
+        tree_a.pull(tree_b.branch, stop_revision=NULL_REVISION, overwrite=True)
+        self.assertEquals(root_id, tree_a.get_root_id())
+
     def test_pull(self):
         tree_a, tree_b = self.get_pullable_trees()
         tree_b.pull(tree_a.branch)
@@ -40,7 +46,7 @@ class TestPull(per_workingtree.TestCaseWithWorkingTree):
     def test_pull_overwrites(self):
         tree_a, tree_b = self.get_pullable_trees()
         tree_b.commit('foo', rev_id='B')
-        self.assertEqual(['B'], tree_b.branch.revision_history())
+        self.assertEqual('B', tree_b.branch.last_revision())
         tree_b.pull(tree_a.branch, overwrite=True)
         self.assertTrue(tree_b.branch.repository.has_revision('A'))
         self.assertTrue(tree_b.branch.repository.has_revision('B'))
@@ -85,10 +91,10 @@ class TestPullWithOrphans(per_workingtree.TestCaseWithWorkingTree):
         return builder.get_branch()
 
     def test_pull_orphans(self):
-        from bzrlib import workingtree
-        if isinstance(self.workingtree_format, workingtree.WorkingTreeFormat2):
+        if not self.workingtree_format.missing_parent_conflicts:
             raise tests.TestSkipped(
-                'WorkingTreeFormat2 does not support missing parent conflicts')
+                '%r does not support missing parent conflicts' %
+                    self.workingtree_format)
         trunk = self.make_branch_deleting_dir('trunk')
         work = trunk.bzrdir.sprout('work', revision_id='2').open_workingtree()
         work.branch.get_config().set_user_option(
