@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -41,8 +41,7 @@ class TestGetBugURL(TestCaseWithMemoryTransport):
         self.tracker_type = TestGetBugURL.TransientTracker
         self.tracker_type.log = []
         bugtracker.tracker_registry.register('transient', self.tracker_type)
-        self.addCleanup(lambda:
-                        bugtracker.tracker_registry.remove('transient'))
+        self.addCleanup(bugtracker.tracker_registry.remove, 'transient')
 
     def test_get_bug_url_for_transient_tracker(self):
         branch = self.make_branch('some_branch')
@@ -118,6 +117,14 @@ class TestBuiltinTrackers(TestCaseWithMemoryTransport):
         self.assertEqual('http://bugs.com/1234/view.html',
                          tracker.get_bug_url('1234'))
 
+    def test_generic_registered_non_integer(self):
+        branch = self.make_branch('some_branch')
+        config = branch.get_config()
+        config.set_user_option('bugtracker_foo_url', 'http://bugs.com/{id}/view.html')
+        tracker = bugtracker.tracker_registry.get_tracker('foo', branch)
+        self.assertEqual('http://bugs.com/ABC-1234/view.html',
+                         tracker.get_bug_url('ABC-1234'))
+
     def test_generic_incorrect_url(self):
         branch = self.make_branch('some_branch')
         config = branch.get_config()
@@ -174,15 +181,13 @@ class TestUniqueIntegerBugTracker(TestCaseWithMemoryTransport):
         self.assertRaises(
             errors.MalformedBugIdentifier, tracker.check_bug_id, 'red')
 
-
-class TestURLParametrizedIntegerBugTracker(TestCaseWithMemoryTransport):
-    """Tests for TracTracker."""
+class TestURLParametrizedBugTracker(TestCaseWithMemoryTransport):
+    """Tests for URLParametrizedBugTracker."""
 
     def setUp(self):
         TestCaseWithMemoryTransport.setUp(self)
         self.url = 'http://twistedmatrix.com/trac'
-        self.tracker = bugtracker.URLParametrizedIntegerBugTracker('some',
-                                                                   'ticket/')
+        self.tracker = bugtracker.URLParametrizedBugTracker('some', 'ticket/')
 
     def test_get_with_unsupported_tag(self):
         """If asked for an unrecognized or unconfigured tag, return None."""
@@ -194,7 +199,7 @@ class TestURLParametrizedIntegerBugTracker(TestCaseWithMemoryTransport):
         """If asked for a valid tag, return a tracker instance that can map bug
         IDs to <base_url>/<bug_area> + <bug_id>."""
         bugtracker.tracker_registry.register('some', self.tracker)
-        self.addCleanup(lambda: bugtracker.tracker_registry.remove('some'))
+        self.addCleanup(bugtracker.tracker_registry.remove, 'some')
 
         branch = self.make_branch('some_branch')
         config = branch.get_config()
@@ -204,6 +209,22 @@ class TestURLParametrizedIntegerBugTracker(TestCaseWithMemoryTransport):
             urlutils.join(self.url, 'ticket/') + '1234',
             tracker.get_bug_url('1234'))
 
+    def test_get_bug_url_for_integer_id(self):
+        self.tracker.check_bug_id('1234')
+
+    def test_get_bug_url_for_non_integer_id(self):
+        self.tracker.check_bug_id('ABC-1234')
+
+
+class TestURLParametrizedIntegerBugTracker(TestCaseWithMemoryTransport):
+    """Tests for URLParametrizedIntegerBugTracker."""
+
+    def setUp(self):
+        TestCaseWithMemoryTransport.setUp(self)
+        self.url = 'http://twistedmatrix.com/trac'
+        self.tracker = bugtracker.URLParametrizedIntegerBugTracker('some',
+                                                                   'ticket/')
+
     def test_get_bug_url_for_bad_bug(self):
         """When given a bug identifier that is invalid for Trac, get_bug_url
         should raise an error.
