@@ -62,34 +62,12 @@ example_conflicts = conflicts.ConflictList(
 ])
 
 
-class TestConflicts(tests.TestCaseWithTransport):
+def vary_by_conflicts():
+    for conflict in example_conflicts:
+        yield (conflict.__class__.__name__, {"conflict": conflict})
 
-    def test_conflicts(self):
-        """Conflicts are detected properly"""
-        # Use BzrDirFormat6 so we can fake conflicts
-        tree = self.make_branch_and_tree('.', format=bzrdir.BzrDirFormat6())
-        self.build_tree_contents([('hello', 'hello world4'),
-                                  ('hello.THIS', 'hello world2'),
-                                  ('hello.BASE', 'hello world1'),
-                                  ('hello.OTHER', 'hello world3'),
-                                  ('hello.sploo.BASE', 'yellowworld'),
-                                  ('hello.sploo.OTHER', 'yellowworld2'),
-                                  ])
-        tree.lock_read()
-        self.assertLength(6, list(tree.list_files()))
-        tree.unlock()
-        tree_conflicts = tree.conflicts()
-        self.assertLength(2, tree_conflicts)
-        self.assertTrue('hello' in tree_conflicts[0].path)
-        self.assertTrue('hello.sploo' in tree_conflicts[1].path)
-        conflicts.restore('hello')
-        conflicts.restore('hello.sploo')
-        self.assertLength(0, tree.conflicts())
-        self.assertFileEqual('hello world2', 'hello')
-        self.assertFalse(os.path.lexists('hello.sploo'))
-        self.assertRaises(errors.NotConflicted, conflicts.restore, 'hello')
-        self.assertRaises(errors.NotConflicted,
-                          conflicts.restore, 'hello.sploo')
+
+class TestConflicts(tests.TestCaseWithTransport):
 
     def test_resolve_conflict_dir(self):
         tree = self.make_branch_and_tree('.')
@@ -147,38 +125,57 @@ class TestConflicts(tests.TestCaseWithTransport):
         self.assertEqual(conflicts.ConflictList([]), tree.conflicts())
 
 
-class TestConflictStanzas(tests.TestCase):
+class TestPerConflict(tests.TestCase):
+
+    scenarios = scenarios.multiply_scenarios(vary_by_conflicts())
+
+    def test_stringification(self):
+        text = unicode(self.conflict)
+        self.assertContainsString(text, self.conflict.path)
+        self.assertContainsString(text.lower(), "conflict")
+        self.assertContainsString(repr(self.conflict),
+            self.conflict.__class__.__name__)
 
     def test_stanza_roundtrip(self):
-        # write and read our example stanza.
-        stanza_iter = example_conflicts.to_stanzas()
-        processed = conflicts.ConflictList.from_stanzas(stanza_iter)
-        for o, p in zip(processed, example_conflicts):
-            self.assertEqual(o, p)
+        p = self.conflict
+        o = conflicts.Conflict.factory(**p.as_stanza().as_dict())
+        self.assertEqual(o, p)
 
-            self.assertIsInstance(o.path, unicode)
+        self.assertIsInstance(o.path, unicode)
 
-            if o.file_id is not None:
-                self.assertIsInstance(o.file_id, str)
+        if o.file_id is not None:
+            self.assertIsInstance(o.file_id, str)
 
-            conflict_path = getattr(o, 'conflict_path', None)
-            if conflict_path is not None:
-                self.assertIsInstance(conflict_path, unicode)
+        conflict_path = getattr(o, 'conflict_path', None)
+        if conflict_path is not None:
+            self.assertIsInstance(conflict_path, unicode)
 
-            conflict_file_id = getattr(o, 'conflict_file_id', None)
-            if conflict_file_id is not None:
-                self.assertIsInstance(conflict_file_id, str)
+        conflict_file_id = getattr(o, 'conflict_file_id', None)
+        if conflict_file_id is not None:
+            self.assertIsInstance(conflict_file_id, str)
 
     def test_stanzification(self):
-        for stanza in example_conflicts.to_stanzas():
-            if 'file_id' in stanza:
-                # In Stanza form, the file_id has to be unicode.
-                self.assertStartsWith(stanza['file_id'], u'\xeed')
-            self.assertStartsWith(stanza['path'], u'p\xe5th')
-            if 'conflict_path' in stanza:
-                self.assertStartsWith(stanza['conflict_path'], u'p\xe5th')
-            if 'conflict_file_id' in stanza:
-                self.assertStartsWith(stanza['conflict_file_id'], u'\xeed')
+        stanza = self.conflict.as_stanza()
+        if 'file_id' in stanza:
+            # In Stanza form, the file_id has to be unicode.
+            self.assertStartsWith(stanza['file_id'], u'\xeed')
+        self.assertStartsWith(stanza['path'], u'p\xe5th')
+        if 'conflict_path' in stanza:
+            self.assertStartsWith(stanza['conflict_path'], u'p\xe5th')
+        if 'conflict_file_id' in stanza:
+            self.assertStartsWith(stanza['conflict_file_id'], u'\xeed')
+
+
+class TestConflictList(tests.TestCase):
+
+    def test_stanzas_roundtrip(self):
+        stanzas_iter = example_conflicts.to_stanzas()
+        processed = conflicts.ConflictList.from_stanzas(stanzas_iter)
+        self.assertEqual(example_conflicts, processed)
+
+    def test_stringification(self):
+        for text, o in zip(example_conflicts.to_strings(), example_conflicts):
+            self.assertEqual(text, unicode(o))
 
 
 # FIXME: The shell-like tests should be converted to real whitebox tests... or
@@ -449,6 +446,14 @@ class TestResolveContentsConflict(TestParametrizedResolveConflicts):
               dict(actions='modify_file', check='file_has_more_content')),
              ('file_deleted',
               dict(actions='delete_file', check='file_doesnt_exist')),),
+            # File renamed-modified/deleted
+            (dict(_base_actions='create_file',
+                  _path='new-file', _file_id='file-id'),
+             ('file_renamed_and_modified',
+              dict(actions='modify_and_rename_file',
+                   check='file_renamed_and_more_content')),
+             ('file_deleted',
+              dict(actions='delete_file', check='file_doesnt_exist')),),
             # File modified/deleted in dir
             (dict(_base_actions='create_file_in_dir',
                   _path='dir/file', _file_id='file-id'),
@@ -466,9 +471,16 @@ class TestResolveContentsConflict(TestParametrizedResolveConflicts):
     def do_modify_file(self):
         return [('modify', ('file-id', 'trunk content\nmore content\n'))]
 
+    def do_modify_and_rename_file(self):
+        return [('modify', ('file-id', 'trunk content\nmore content\n')),
+                ('rename', ('file', 'new-file'))]
+
     def check_file_has_more_content(self):
         self.assertFileEqual('trunk content\nmore content\n', 'branch/file')
 
+    def check_file_renamed_and_more_content(self):
+        self.assertFileEqual('trunk content\nmore content\n', 'branch/new-file')
+
     def do_delete_file(self):
         return [('unversion', 'file-id')]
 
@@ -665,6 +677,14 @@ class TestResolveDuplicateEntry(TestParametrizedResolveConflicts):
              ('fileb_created',
               dict(actions='create_file_b', check='file_content_b',
                    path='file', file_id='file-b-id')),),
+            # File created with different file-ids but deleted on one side
+            (dict(_base_actions='create_file_a'),
+             ('filea_replaced',
+              dict(actions='replace_file_a_by_b', check='file_content_b',
+                   path='file', file_id='file-b-id')),
+             ('filea_modified',
+              dict(actions='modify_file_a', check='file_new_content',
+                   path='file', file_id='file-a-id')),),
             ])
 
     def do_nothing(self):
@@ -682,6 +702,16 @@ class TestResolveDuplicateEntry(TestParametrizedResolveConflicts):
     def check_file_content_b(self):
         self.assertFileEqual('file b content\n', 'branch/file')
 
+    def do_replace_file_a_by_b(self):
+        return [('unversion', 'file-a-id'),
+                ('add', ('file', 'file-b-id', 'file', 'file b content\n'))]
+
+    def do_modify_file_a(self):
+        return [('modify', ('file-a-id', 'new content\n'))]
+
+    def check_file_new_content(self):
+        self.assertFileEqual('new content\n', 'branch/file')
+
     def _get_resolve_path_arg(self, wt, action):
         return self._this['path']
 
@@ -729,14 +759,14 @@ $ bzr merge ../trunk
         self.run_script("""
 $ bzr rm -q dir  --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_take_other(self):
         self.run_script("""
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -770,7 +800,7 @@ $ bzr merge ../trunk
     def test_keep_them_all(self):
         self.run_script("""
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -779,7 +809,7 @@ $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 $ bzr mv -q dir/file2 file2
 $ bzr rm -q dir --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -787,7 +817,7 @@ $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
         self.run_script("""
 $ bzr rm -q dir --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -834,7 +864,7 @@ $ bzr merge ../trunk
     def test_keep_them_all(self):
         self.run_script("""
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -843,7 +873,7 @@ $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 $ bzr mv -q dir/file2 file2
 $ bzr rm -q dir --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -851,14 +881,14 @@ $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
         self.run_script("""
 $ bzr rm -q dir --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_resolve_taking_this(self):
         self.run_script("""
 $ bzr resolve --take-this dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -867,7 +897,7 @@ $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 $ bzr resolve --take-other dir
 2>deleted dir/file2
 2>deleted dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -959,7 +989,7 @@ class TestResolveParentLoop(TestParametrizedResolveConflicts):
         if self._other['xfail']:
             # It's a bit hackish to raise from here relying on being called for
             # both tests but this avoid overriding test_resolve_taking_other
-            raise tests.KnownFailure(
+            self.knownFailure(
                 "ParentLoop doesn't carry enough info to resolve --take-other")
     _assert_conflict = assertParentLoop
 
@@ -997,7 +1027,7 @@ $ bzr rm -q foo.new --force
 # aside ? -- vila 090916
 $ bzr add -q foo
 $ bzr resolve foo.new
-2>1 conflict(s) resolved, 0 remaining
+2>1 conflict resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -1006,7 +1036,7 @@ $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 $ bzr rm -q foo --force
 $ bzr mv -q foo.new foo
 $ bzr resolve foo
-2>1 conflict(s) resolved, 0 remaining
+2>1 conflict resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -1031,7 +1061,8 @@ class TestMalformedTransform(script.TestCaseWithTransportAndScript):
         # This is nearly like TestResolveNonDirectoryParent but with branch and
         # trunk switched. As such it should certainly produce the same
         # conflict.
-        self.run_script("""
+        self.assertRaises(errors.MalformedTransform,
+                          self.run_script,"""
 $ bzr init trunk
 ...
 $ cd trunk
@@ -1068,7 +1099,7 @@ $ bzr commit -m 'create file on trunk'
 # Create a debian branch based on trunk
 $ cd ..
 $ bzr branch trunk -r 1 debian
-2>Branched 1 revision(s).
+2>Branched 1 revision.
 $ cd debian
 $ mkdir dir
 $ bzr add
@@ -1085,6 +1116,15 @@ $ cd ..
 $ bzr init experimental
 Created a standalone tree (format: 2a)
 $ cd experimental
+# Work around merging into empty branch not being supported
+# (http://pad.lv/308562)
+$ echo something >not-empty
+$ bzr add
+adding not-empty
+$ bzr commit -m 'Add some content in experimental'
+2>Committing to: .../experimental/
+2>added not-empty
+2>Committed revision 1.
 # merge debian even without a common ancestor
 $ bzr merge ../debian -r0..2
 2>+N  dir/
@@ -1092,14 +1132,23 @@ $ bzr merge ../debian -r0..2
 2>All changes applied successfully.
 $ bzr commit -m 'merging debian into experimental'
 2>Committing to: .../experimental/
-2>deleted 
-2>modified dir
-2>Committed revision 1.
+2>added dir
+2>added dir/file
+2>Committed revision 2.
 # Create an ubuntu branch with yet another root-id
 $ cd ..
 $ bzr init ubuntu
 Created a standalone tree (format: 2a)
 $ cd ubuntu
+# Work around merging into empty branch not being supported
+# (http://pad.lv/308562)
+$ echo something >not-empty-ubuntu
+$ bzr add
+adding not-empty-ubuntu
+$ bzr commit -m 'Add some content in experimental'
+2>Committing to: .../ubuntu/
+2>added not-empty-ubuntu
+2>Committed revision 1.
 # Also merge debian
 $ bzr merge ../debian -r0..2
 2>+N  dir/
@@ -1107,11 +1156,12 @@ $ bzr merge ../debian -r0..2
 2>All changes applied successfully.
 $ bzr commit -m 'merging debian'
 2>Committing to: .../ubuntu/
-2>deleted 
-2>modified dir
-2>Committed revision 1.
+2>added dir
+2>added dir/file
+2>Committed revision 2.
 # Now try to merge experimental
 $ bzr merge ../experimental
+2>+N  not-empty
 2>Path conflict: dir / dir
 2>1 conflicts encountered.
 """)
