@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,11 +18,9 @@
 
 import codecs
 import locale
-import os
 import sys
 
 from bzrlib import (
-    errors,
     osutils,
     )
 from bzrlib.tests import (
@@ -75,12 +73,10 @@ class TestTerminalEncoding(TestCase):
 
     def setUp(self):
         TestCase.setUp(self)
-        self._stdout = sys.stdout
-        self._stderr = sys.stderr
-        self._stdin = sys.stdin
-        self._user_encoding = osutils._cached_user_encoding
-
-        self.addCleanup(self._reset)
+        self.overrideAttr(sys, 'stdin')
+        self.overrideAttr(sys, 'stdout')
+        self.overrideAttr(sys, 'stderr')
+        self.overrideAttr(osutils, '_cached_user_encoding')
 
     def make_wrapped_streams(self,
                              stdout_encoding,
@@ -100,12 +96,6 @@ class TestTerminalEncoding(TestCase):
             fake_codec.add(stderr_encoding)
             fake_codec.add(stdin_encoding)
 
-    def _reset(self):
-        sys.stdout = self._stdout
-        sys.stderr = self._stderr
-        sys.stdin = self._stdin
-        osutils._cached_user_encoding = self._user_encoding
-
     def test_get_terminal_encoding(self):
         self.make_wrapped_streams('stdout_encoding',
                                   'stderr_encoding',
@@ -122,6 +112,26 @@ class TestTerminalEncoding(TestCase):
         # and in the worst case, use osutils.get_user_encoding()
         self.assertEqual('user_encoding', osutils.get_terminal_encoding())
 
+    def test_get_terminal_encoding_silent(self):
+        self.make_wrapped_streams('stdout_encoding',
+                                  'stderr_encoding',
+                                  'stdin_encoding')
+        # Calling get_terminal_encoding should not mutter when silent=True is
+        # passed.
+        log = self.get_log()
+        osutils.get_terminal_encoding()
+        self.assertEqual(log, self.get_log())
+
+    def test_get_terminal_encoding_trace(self):
+        self.make_wrapped_streams('stdout_encoding',
+                                  'stderr_encoding',
+                                  'stdin_encoding')
+        # Calling get_terminal_encoding should not mutter when silent=True is
+        # passed.
+        log = self.get_log()
+        osutils.get_terminal_encoding(trace=True)
+        self.assertNotEqual(log, self.get_log())
+
     def test_terminal_cp0(self):
         # test cp0 encoding (Windows returns cp0 when there is no encoding)
         self.make_wrapped_streams('cp0',
@@ -158,65 +168,35 @@ class TestUserEncoding(TestCase):
 
     def setUp(self):
         TestCase.setUp(self)
-        self._stderr = sys.stderr
-        self._getpreferredencoding = locale.getpreferredencoding
-        self.addCleanup(self._reset)
-        sys.stderr = StringIOWrapper()
-        # save $LANG
-        self._LANG = os.environ.get('LANG')
+        self.overrideAttr(osutils, '_cached_user_encoding', None)
+        self.overrideAttr(locale, 'getpreferredencoding', self.get_encoding)
+        self.overrideAttr(locale, 'CODESET', None)
+        self.overrideAttr(sys, 'stderr', StringIOWrapper())
 
-    def _reset(self):
-        locale.getpreferredencoding = self._getpreferredencoding
-        sys.stderr = self._stderr
-        # restore $LANG
-        osutils.set_or_unset_env('LANG', self._LANG)
+    def get_encoding(self, do_setlocale=True):
+        return self._encoding
 
     def test_get_user_encoding(self):
-        def f():
-            return 'user_encoding'
-
-        locale.getpreferredencoding = f
+        self._encoding = 'user_encoding'
         fake_codec.add('user_encoding')
-        self.assertEquals('user_encoding', osutils.get_user_encoding(use_cache=False))
+        self.assertEquals('iso8859-1', # fake_codec maps to latin-1
+                          osutils.get_user_encoding())
         self.assertEquals('', sys.stderr.getvalue())
 
     def test_user_cp0(self):
-        def f():
-            return 'cp0'
-
-        locale.getpreferredencoding = f
-        self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
+        self._encoding = 'cp0'
+        self.assertEquals('ascii', osutils.get_user_encoding())
         self.assertEquals('', sys.stderr.getvalue())
 
     def test_user_cp_unknown(self):
-        def f():
-            return 'cp-unknown'
-
-        locale.getpreferredencoding = f
-        self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
+        self._encoding = 'cp-unknown'
+        self.assertEquals('ascii', osutils.get_user_encoding())
         self.assertEquals('bzr: warning: unknown encoding cp-unknown.'
                           ' Continuing with ascii encoding.\n',
                           sys.stderr.getvalue())
 
     def test_user_empty(self):
         """Running bzr from a vim script gives '' for a preferred locale"""
-        def f():
-            return ''
-
-        locale.getpreferredencoding = f
-        self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
+        self._encoding = ''
+        self.assertEquals('ascii', osutils.get_user_encoding())
         self.assertEquals('', sys.stderr.getvalue())
-
-    def test_user_locale_error(self):
-        def f():
-            raise locale.Error, 'unsupported locale'
-
-        locale.getpreferredencoding = f
-        os.environ['LANG'] = 'BOGUS'
-        self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
-        self.assertEquals('bzr: warning: unsupported locale\n'
-                          '  Could not determine what text encoding to use.\n'
-                          '  This error usually means your Python interpreter\n'
-                          '  doesn\'t support the locale set by $LANG (BOGUS)\n'
-                          '  Continuing with ascii encoding.\n',
-                          sys.stderr.getvalue())
