@@ -15,14 +15,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
-
 from bzrlib import (
     bzrdir,
     conflicts,
     errors,
+    symbol_versioning,
     transport,
     workingtree,
+    workingtree_3,
+    workingtree_4,
     )
 from bzrlib.lockdir import LockDir
 from bzrlib.mutabletree import needs_tree_write_lock
@@ -62,10 +63,10 @@ class TestTreeLink(TestCaseWithTransport):
 class TestDefaultFormat(TestCaseWithTransport):
 
     def test_get_set_default_format(self):
-        old_format = workingtree.WorkingTreeFormat.get_default_format()
-        # default is 3
-        self.assertTrue(isinstance(old_format, workingtree.WorkingTreeFormat3))
-        workingtree.WorkingTreeFormat.set_default_format(SampleTreeFormat())
+        old_format = workingtree.format_registry.get_default()
+        # default is 6
+        self.assertTrue(isinstance(old_format, workingtree_4.WorkingTreeFormat6))
+        workingtree.format_registry.set_default(SampleTreeFormat())
         try:
             # the default branch format is used by the meta dir format
             # which is not the default bzrdir format at this point
@@ -75,8 +76,36 @@ class TestDefaultFormat(TestCaseWithTransport):
             result = dir.create_workingtree()
             self.assertEqual(result, 'A tree')
         finally:
-            workingtree.WorkingTreeFormat.set_default_format(old_format)
-        self.assertEqual(old_format, workingtree.WorkingTreeFormat.get_default_format())
+            workingtree.format_registry.set_default(old_format)
+        self.assertEqual(old_format, workingtree.format_registry.get_default())
+
+    def test_from_string(self):
+        self.assertIsInstance(
+            SampleTreeFormat.from_string("Sample tree format."),
+            SampleTreeFormat)
+        self.assertRaises(AssertionError,
+            SampleTreeFormat.from_string, "Different format string.")
+
+    def test_get_set_default_format_by_key(self):
+        old_format = workingtree.format_registry.get_default()
+        # default is 6
+        format = SampleTreeFormat()
+        workingtree.format_registry.register(format)
+        self.addCleanup(workingtree.format_registry.remove, format)
+        self.assertTrue(isinstance(old_format, workingtree_4.WorkingTreeFormat6))
+        workingtree.format_registry.set_default_key(format.get_format_string())
+        try:
+            # the default branch format is used by the meta dir format
+            # which is not the default bzrdir format at this point
+            dir = bzrdir.BzrDirMetaFormat1().initialize('.')
+            dir.create_repository()
+            dir.create_branch()
+            result = dir.create_workingtree()
+            self.assertEqual(result, 'A tree')
+        finally:
+            workingtree.format_registry.set_default_key(
+                old_format.get_format_string())
+        self.assertEqual(old_format, workingtree.format_registry.get_default())
 
     def test_open(self):
         tree = self.make_branch_and_tree('.')
@@ -98,14 +127,15 @@ class TestDefaultFormat(TestCaseWithTransport):
         self.assertEqual('subdir', relpath)
 
 
-class SampleTreeFormat(workingtree.WorkingTreeFormat):
+class SampleTreeFormat(workingtree.WorkingTreeFormatMetaDir):
     """A sample format
 
     this format is initializable, unsupported to aid in testing the
     open and open_downlevel routines.
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Sample tree format."
 
@@ -123,9 +153,42 @@ class SampleTreeFormat(workingtree.WorkingTreeFormat):
         return "opened tree."
 
 
+class SampleExtraTreeFormat(workingtree.WorkingTreeFormat):
+    """A sample format that does not support use in a metadir.
+
+    """
+
+    def get_format_string(self):
+        # Not usable in a metadir, so no format string
+        return None
+
+    def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
+                   accelerator_tree=None, hardlink=False):
+        raise NotImplementedError(self.initialize)
+
+    def is_supported(self):
+        return False
+
+    def open(self, transport, _found=False):
+        raise NotImplementedError(self.open)
+
+
 class TestWorkingTreeFormat(TestCaseWithTransport):
     """Tests for the WorkingTreeFormat facility."""
 
+    def test_find_format_string(self):
+        # is the right format object found for a working tree?
+        branch = self.make_branch('branch')
+        self.assertRaises(errors.NoWorkingTree,
+            workingtree.WorkingTreeFormatMetaDir.find_format_string, branch.bzrdir)
+        transport = branch.bzrdir.get_workingtree_transport(None)
+        transport.mkdir('.')
+        transport.put_bytes("format", "some format name")
+        # The format does not have to be known by Bazaar,
+        # find_format_string just retrieves the name
+        self.assertEquals("some format name",
+            workingtree.WorkingTreeFormatMetaDir.find_format_string(branch.bzrdir))
+
     def test_find_format(self):
         # is the right format object found for a working tree?
         # create a branch with a few known format objects.
@@ -136,14 +199,14 @@ class TestWorkingTreeFormat(TestCaseWithTransport):
             dir.create_branch()
             format.initialize(dir)
             t = transport.get_transport(url)
-            found_format = workingtree.WorkingTreeFormat.find_format(dir)
+            found_format = workingtree.WorkingTreeFormatMetaDir.find_format(dir)
             self.assertIsInstance(found_format, format.__class__)
-        check_format(workingtree.WorkingTreeFormat3(), "bar")
+        check_format(workingtree_3.WorkingTreeFormat3(), "bar")
 
     def test_find_format_no_tree(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize('.')
         self.assertRaises(errors.NoWorkingTree,
-                          workingtree.WorkingTreeFormat.find_format,
+                          workingtree.WorkingTreeFormatMetaDir.find_format,
                           dir)
 
     def test_find_format_unknown_format(self):
@@ -152,7 +215,7 @@ class TestWorkingTreeFormat(TestCaseWithTransport):
         dir.create_branch()
         SampleTreeFormat().initialize(dir)
         self.assertRaises(errors.UnknownFormatError,
-                          workingtree.WorkingTreeFormat.find_format,
+                          workingtree.WorkingTreeFormatMetaDir.find_format,
                           dir)
 
     def test_register_unregister_format(self):
@@ -164,13 +227,110 @@ class TestWorkingTreeFormat(TestCaseWithTransport):
         # make a branch
         format.initialize(dir)
         # register a format for it.
-        workingtree.WorkingTreeFormat.register_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            workingtree.WorkingTreeFormat.register_format, format)
+        self.assertTrue(format in 
+            self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+                workingtree.WorkingTreeFormat.get_formats))
         # which branch.Open will refuse (not supported)
         self.assertRaises(errors.UnsupportedFormatError, workingtree.WorkingTree.open, '.')
         # but open_downlevel will work
         self.assertEqual(format.open(dir), workingtree.WorkingTree.open_downlevel('.'))
         # unregister the format
-        workingtree.WorkingTreeFormat.unregister_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            workingtree.WorkingTreeFormat.unregister_format, format)
+        self.assertFalse(format in
+            self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+                workingtree.WorkingTreeFormat.get_formats))
+
+    def test_find_format_with_features(self):
+        tree = self.make_branch_and_tree('.', format='2a')
+        tree.update_feature_flags({"name": "necessity"})
+        found_format = workingtree.WorkingTreeFormatMetaDir.find_format(
+            tree.bzrdir)
+        self.assertIsInstance(found_format, workingtree.WorkingTreeFormat)
+        self.assertEquals(found_format.features.get("name"), "necessity")
+        self.assertRaises(errors.MissingFeature, found_format.check_support_status,
+            True)
+        self.addCleanup(workingtree.WorkingTreeFormatMetaDir.unregister_feature,
+            "name")
+        workingtree.WorkingTreeFormatMetaDir.register_feature("name")
+        found_format.check_support_status(True)
+
+
+class TestWorkingTreeIterEntriesByDir_wSubtrees(TestCaseWithTransport):
+
+    def make_simple_tree(self):
+        tree = self.make_branch_and_tree('tree', format='development-subtree')
+        self.build_tree(['tree/a/', 'tree/a/b/', 'tree/a/b/c'])
+        tree.set_root_id('root-id')
+        tree.add(['a', 'a/b', 'a/b/c'], ['a-id', 'b-id', 'c-id'])
+        tree.commit('initial')
+        return tree
+
+    def test_just_directory(self):
+        tree = self.make_simple_tree()
+        self.assertEqual([('directory', 'root-id'),
+                          ('directory', 'a-id'),
+                          ('directory', 'b-id'),
+                          ('file', 'c-id')],
+                         [(ie.kind, ie.file_id)
+                          for path, ie in tree.iter_entries_by_dir()])
+        subtree = self.make_branch_and_tree('tree/a/b')
+        self.assertEqual([('tree-reference', 'b-id')],
+                         [(ie.kind, ie.file_id)
+                          for path, ie in tree.iter_entries_by_dir(['b-id'])])
+
+    def test_direct_subtree(self):
+        tree = self.make_simple_tree()
+        subtree = self.make_branch_and_tree('tree/a/b')
+        self.assertEqual([('directory', 'root-id'),
+                          ('directory', 'a-id'),
+                          ('tree-reference', 'b-id')],
+                         [(ie.kind, ie.file_id)
+                          for path, ie in tree.iter_entries_by_dir()])
+
+    def test_indirect_subtree(self):
+        tree = self.make_simple_tree()
+        subtree = self.make_branch_and_tree('tree/a')
+        self.assertEqual([('directory', 'root-id'),
+                          ('tree-reference', 'a-id')],
+                         [(ie.kind, ie.file_id)
+                          for path, ie in tree.iter_entries_by_dir()])
+
+
+class TestWorkingTreeFormatRegistry(TestCase):
+
+    def setUp(self):
+        super(TestWorkingTreeFormatRegistry, self).setUp()
+        self.registry = workingtree.WorkingTreeFormatRegistry()
+
+    def test_register_unregister_format(self):
+        format = SampleTreeFormat()
+        self.registry.register(format)
+        self.assertEquals(format, self.registry.get("Sample tree format."))
+        self.registry.remove(format)
+        self.assertRaises(KeyError, self.registry.get, "Sample tree format.")
+
+    def test_get_all(self):
+        format = SampleTreeFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra(self):
+        format = SampleExtraTreeFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra_lazy(self):
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra_lazy("bzrlib.tests.test_workingtree",
+            "SampleExtraTreeFormat")
+        formats = self.registry._get_all()
+        self.assertEquals(1, len(formats))
+        self.assertIsInstance(formats[0], SampleExtraTreeFormat)
 
 
 class TestWorkingTreeFormat3(TestCaseWithTransport):
@@ -180,7 +340,7 @@ class TestWorkingTreeFormat3(TestCaseWithTransport):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         control.create_repository()
         control.create_branch()
-        tree = workingtree.WorkingTreeFormat3().initialize(control)
+        tree = workingtree_3.WorkingTreeFormat3().initialize(control)
         # we want:
         # format 'Bazaar-NG Working Tree format 3'
         # inventory = blank inventory
@@ -214,7 +374,7 @@ class TestWorkingTreeFormat3(TestCaseWithTransport):
         repo = dir.create_repository()
         branch = dir.create_branch()
         try:
-            tree = workingtree.WorkingTreeFormat3().initialize(dir)
+            tree = workingtree_3.WorkingTreeFormat3().initialize(dir)
         except errors.NotLocalUrl:
             raise TestSkipped('Not a local URL')
         self.assertIsDirectory('.bzr', t)
@@ -231,41 +391,11 @@ class TestWorkingTreeFormat3(TestCaseWithTransport):
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         control.create_repository()
         control.create_branch()
-        tree = workingtree.WorkingTreeFormat3().initialize(control)
+        tree = workingtree_3.WorkingTreeFormat3().initialize(control)
         tree._transport.delete("pending-merges")
         self.assertEqual([], tree.get_parent_ids())
 
 
-class TestFormat2WorkingTree(TestCaseWithTransport):
-    """Tests that are specific to format 2 trees."""
-
-    def create_format2_tree(self, url):
-        return self.make_branch_and_tree(
-            url, format=bzrdir.BzrDirFormat6())
-
-    def test_conflicts(self):
-        # test backwards compatability
-        tree = self.create_format2_tree('.')
-        self.assertRaises(errors.UnsupportedOperation, tree.set_conflicts,
-                          None)
-        file('lala.BASE', 'wb').write('labase')
-        expected = conflicts.ContentsConflict('lala')
-        self.assertEqual(list(tree.conflicts()), [expected])
-        file('lala', 'wb').write('la')
-        tree.add('lala', 'lala-id')
-        expected = conflicts.ContentsConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflicts()), [expected])
-        file('lala.THIS', 'wb').write('lathis')
-        file('lala.OTHER', 'wb').write('laother')
-        # When "text conflict"s happen, stem, THIS and OTHER are text
-        expected = conflicts.TextConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflicts()), [expected])
-        os.unlink('lala.OTHER')
-        os.mkdir('lala.OTHER')
-        expected = conflicts.ContentsConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflicts()), [expected])
-
-
 class InstrumentedTree(object):
     """A instrumented tree to check the needs_tree_write_lock decorator."""
 
