@@ -23,16 +23,14 @@ Written by Mattias Eriksson <snaggen@acc.umu.se> based on the ftp transport.
 It provides the gio+XXX:// protocols where XXX is any of the protocols
 supported by gio.
 """
+
+from __future__ import absolute_import
+
 from cStringIO import StringIO
-import getpass
 import os
 import random
-import socket
 import stat
-import urllib
 import time
-import sys
-import getpass
 import urlparse
 
 from bzrlib import (
@@ -43,12 +41,17 @@ from bzrlib import (
     debug,
     ui,
     )
-from bzrlib.trace import mutter, warning
+from bzrlib.symbol_versioning import (
+    DEPRECATED_PARAMETER,
+    deprecated_in,
+    deprecated_passed,
+    warn,
+    )
+from bzrlib.trace import mutter
 from bzrlib.transport import (
     FileStream,
     ConnectedTransport,
     _file_streams,
-    Server,
     )
 
 from bzrlib.tests.test_server import TestServer
@@ -165,35 +168,37 @@ class GioTransport(ConnectedTransport):
         #really use bzrlib.auth get_password for this
         #or possibly better gnome-keyring?
         auth = config.AuthenticationConfig()
-        (scheme, urluser, urlpassword, host, port, urlpath) = \
-           urlutils.parse_url(self.url)
+        parsed_url = urlutils.URL.from_string(self.url)
         user = None
         if (flags & gio.ASK_PASSWORD_NEED_USERNAME and
                 flags & gio.ASK_PASSWORD_NEED_DOMAIN):
-            prompt = scheme.upper() + ' %(host)s DOMAIN\username'
-            user_and_domain = auth.get_user(scheme, host,
-                    port=port, ask=True, prompt=prompt)
+            prompt = (u'%s' % (parsed_url.scheme.upper(),) +
+                      u' %(host)s DOMAIN\\username')
+            user_and_domain = auth.get_user(parsed_url.scheme,
+                parsed_url.host, port=parsed_url.port, ask=True,
+                prompt=prompt)
             (domain, user) = user_and_domain.split('\\', 1)
             op.set_username(user)
             op.set_domain(domain)
         elif flags & gio.ASK_PASSWORD_NEED_USERNAME:
-            user = auth.get_user(scheme, host,
-                    port=port, ask=True)
+            user = auth.get_user(parsed_url.scheme, parsed_url.host,
+                    port=parsed_url.port, ask=True)
             op.set_username(user)
         elif flags & gio.ASK_PASSWORD_NEED_DOMAIN:
             #Don't know how common this case is, but anyway
             #a DOMAIN and a username prompt should be the
             #same so I will missuse the ui_factory get_username
             #a little bit here.
-            prompt = scheme.upper() + ' %(host)s DOMAIN'
+            prompt = (u'%s' % (parsed_url.scheme.upper(),) +
+                      u' %(host)s DOMAIN')
             domain = ui.ui_factory.get_username(prompt=prompt)
             op.set_domain(domain)
 
         if flags & gio.ASK_PASSWORD_NEED_PASSWORD:
             if user is None:
                 user = op.get_username()
-            password = auth.get_password(scheme, host,
-                    user, port=port)
+            password = auth.get_password(parsed_url.scheme, parsed_url.host,
+                    user, port=parsed_url.port)
             op.set_password(password)
         op.reply(gio.MOUNT_OPERATION_HANDLED)
 
@@ -207,7 +212,7 @@ class GioTransport(ConnectedTransport):
 
     def _create_connection(self, credentials=None):
         if credentials is None:
-            user, password = self._user, self._password
+            user, password = self._parsed_url.user, self._parsed_url.password
         else:
             user, password = credentials
 
@@ -235,16 +240,20 @@ class GioTransport(ConnectedTransport):
                                         " %s" % str(e), orig_error=e)
         return connection, (user, password)
 
+    def disconnect(self):
+        # FIXME: Nothing seems to be necessary here, which sounds a bit strange
+        # -- vila 20100601
+        pass
+
     def _reconnect(self):
+        # FIXME: This doesn't seem to be used -- vila 20100601
         """Create a new connection with the previously used credentials"""
         credentials = self._get_credentials()
         connection, credentials = self._create_connection(credentials)
         self._set_connection(connection, credentials)
 
     def _remote_path(self, relpath):
-        relative = urlutils.unescape(relpath).encode('utf-8')
-        remote_path = self._combine_paths(self._path, relative)
-        return remote_path
+        return self._parsed_url.clone(relpath).path
 
     def has(self, relpath):
         """Does the target location exist?"""
@@ -262,7 +271,7 @@ class GioTransport(ConnectedTransport):
             else:
                 self._translate_gio_error(e, relpath)
 
-    def get(self, relpath, decode=False, retries=0):
+    def get(self, relpath, decode=DEPRECATED_PARAMETER, retries=0):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
@@ -272,6 +281,10 @@ class GioTransport(ConnectedTransport):
         We're meant to return a file-like object which bzr will
         then read from. For now we do this via the magic of StringIO
         """
+        if deprecated_passed(decode):
+            warn(deprecated_in((2,3,0)) %
+                 '"decode" parameter to GioTransport.get()',
+                 DeprecationWarning, stacklevel=2)
         try:
             if 'gio' in debug.debug_flags:
                 mutter("GIO get: %s" % relpath)
